import multiprocessing
from openQCM.core.constants import Constants
from openQCM.common.fileStorage import FileStorage
from openQCM.common.logger import Logger as Log

#from progress.bar import Bar 
from progressbar import Bar, Percentage, ProgressBar, RotatingMarker, Timer

import serial
from serial.tools import list_ports
import numpy as np
import scipy.signal
from numpy import loadtxt


TAG = ""#"[Calibration]"

###############################################################################
# Process for the serial package and the communication with the serial port
# Processes incoming data and calculates outgoing data by the algorithms
###############################################################################
class CalibrationProcess(multiprocessing.Process):
    
    
    ###########################################################################
    # BASELINE ESTIMATION
    # Estimates Baseline with Least Squares Polynomial Fit (LSP)
    ###########################################################################
    def baseline_estimation(self,x,y,poly_order):
        # Least Squares Polynomial Fit (LSP)
        coeffs = np.polyfit(x,y,poly_order) 
        # Evaluate a polynomial at specific values
        poly_fitted = np.polyval(coeffs,x) 
        return poly_fitted,coeffs       
      
        
    ###########################################################################
    # BASELINE CORRECTION
    # estimates signal-baseline for amplitude and phase
    ###########################################################################
    def baseline_correction(self,readFREQ,data_mag,data_ph):
        
        # input signal Amplitude
        (self._polyfitted_all,self._coeffs_all) = self.baseline_estimation(readFREQ,data_mag,8)
        self._mag_beseline_corrected_all = data_mag-self._polyfitted_all
        
        # input signal Phase
        (self._polyfitted_all_phase,self._coeffs_all_phase) = self.baseline_estimation(readFREQ,data_ph,8)
        self._phase_beseline_corrected_all = data_ph - self._polyfitted_all_phase 
        return self._mag_beseline_corrected_all, self._phase_beseline_corrected_all
    
    
    ###########################################################################
    # PEAK DETECTION
    # Calculates the relative extrema of data using Signal Processing Toolbox 
    ###########################################################################    
    def FindPeak(self,freq, mag, phase, dist):
        
        # freq vector of frequencies and mag and phase vectors of of values, 
        # dist is minimal horizontal distance (dist>=1) in samples between neighbouring peaks.
        self.max_indexes_mag = scipy.signal.argrelextrema(np.array(mag),comparator=np.greater,order=dist)   
        self.max_indexes_phase = scipy.signal.argrelextrema(np.array(phase),comparator=np.greater,order=dist)
        
        # local maxima amplitude
        self.max_freq_mag=freq[self.max_indexes_mag]
        self.max_value_mag=mag[self.max_indexes_mag]
        
        # local maxima phase
        self.max_freq_phase=freq[self.max_indexes_phase]
        self.max_value_phase=phase[self.max_indexes_phase]
        
        return self.max_freq_mag, self.max_value_mag, self.max_freq_phase, self.max_value_phase


    ###########################################################################
    # Initializing values for process
    ###########################################################################
    def __init__(self, parser_process):
        """
        :param parser_process: Reference to a ParserProcess instance.
        :type parser_process: ParserProcess.
        """
        multiprocessing.Process.__init__(self)
        self._exit = multiprocessing.Event()
        
        # Instantiate a ParserProcess class for each communication channels
        self._parser1 = parser_process
        self._parser2 = parser_process
        #self._parser3 = parser_process
        #self._parser4 = parser_process
        self._parser5 = parser_process
        self._parser6 = parser_process
        self._serial = serial.Serial()
        
    ###########################################################################
    # Opens a specified serial port
    ###########################################################################    
    def open(self, port, 
                   speed = Constants.serial_default_QCS, 
                   timeout = Constants.serial_timeout_ms, 
                   writeTimeout = Constants.serial_writetimeout_ms):
        """
        :param port: Serial port name :type port: str.
        :param speed: Baud rate, in bps, to connect to port :type speed: int.
        :param timeout: Sets current read timeout :type timeout: float (seconds).
        :param writetTimeout: Sets current write timeout :type writeTimeout: float (seconds).
        :return: True if the port is available :rtype: bool.
        """
        self._serial.port = port
        self._serial.baudrate = Constants.serial_default_speed #115200
        self._serial.stopbits = serial.STOPBITS_ONE
        self._serial.bytesize = serial.EIGHTBITS
        self._serial.timeout = timeout
        self._serial.writetimeout = writeTimeout
        self._QCStype = speed
        
        # Variable to process the exception
        #wrong = False
        # Checks QCStype to calibrate
        if self._QCStype == '@5MHz_QCM':
           self._QCStype_int = 0
        elif self._QCStype =='@10MHz_QCM':
           self._QCStype_int = 1
        #else: 
        #   wrong = True
        #   print(TAG, "Warning: wrong QCM Sensor selected, set default to @5MHz") 
        #   self._QCStype_int = 0
        #if not wrong:
        print(TAG, "Selected Quartz Crystal Sensor:",self._QCStype)
        return self._is_port_available(self._serial.port)
    
    ###########################################################################
    # Reads the serial port,processes and adds all the data to internal queues
    ###########################################################################
    def run(self):
        """
        The expected format is a buffer (sweep) and a new buffer as a new sweep. 
        The method parses data, converts each value to float and adds to a queue. 
        If incoming data can't be converted to float,the data will be discarded.
        """  
        # initializations
        self._polyfitted_all = None
        self._coeffs_all = None
        self._polyfitted_all_phase = None
        self._coeffs_all_phase = None
        self._mag_beseline_corrected_all = None
        self._phase_beseline_corrected_all = None
        self._flag = 0
        self._flag2 = 0
        
        # Checks if the serial port is currently connected
        if self._is_port_available(self._serial.port):
            
            # Sets start, stop, step and range frequencies 
            #startFreq = Constants.calibration_frequency_start
            #stopFreq  = Constants.calibration_frequency_stop
            #samples   = Constants.calibration_default_samples 
            #fStep     = Constants.calibration_fStep
            readFREQ  = Constants.calibration_readFREQ
            # Gets the state of the serial port
            if not self._serial.isOpen(): 
                # Opens the serial port
                self._serial.open() 
                self._serial.flushInput()
                self._serial.flushOutput()
                # Initializes the sweep counter
                k=0 
                print(TAG,'Calibration Process Started')
                print(TAG,'The operation might take just over a minute to complete... please wait...')
                #### SWEEPS LOOP ####
                #----------------------------------------------------------
                temp1=[]
                temp2=[]
                #----------------------------------------------------------
                while not self._exit.is_set():
                    # Boolean variable to process exceptions
                    self._flag = 0
                    self._flag2 = 0
                    fStep = Constants.calib_fStep #1000
                    
                    # Sets start, stop, step and range frequencies 
                    startFreq = Constants.calibration_frequency_start + k*Constants.calib_fRange   #5000000/10000000
                    stopFreq  = startFreq + Constants.calib_fRange #5000000
                    
                    samples   = Constants.calib_samples #5001/10001
                    # data reset for new sweep
                    data_mag = np.linspace(0,0,samples)   
                    data_ph  = np.linspace(0,0,samples)
                    
                    
                    try:
                        # amplitude/phase convert bit to dB/Deg 
                        vmax = 3.3
                        bitmax = 8192 
                        ADCtoVolt = vmax / bitmax
                        VCP = 0.9
                        
                        # WRITES encoded command to the serial port
                        cmd = str(startFreq) + ';' + str(stopFreq) + ';' + str(int(fStep)) + '\n'
                        #print(cmd)
                        self._serial.write(cmd.encode())
                        
                        # Initializes buffer and strs
                        buffer = ''
                        strs = ["" for x in range(samples + 2)]
                        
                        # Initializes the progress bar
                        #################################################################################
                        # CHANGED v2.0
                        # INCREASED maxval=1000000 TO AVOID bar.update(len.buffer) BREAKS THE CALIBRATION  
                        #################################################################################
                        # bar = ProgressBar(widgets=[TAG,' ', Bar(marker='>'),' ',Percentage(),' ', Timer()], maxval=830000).start()
                        # READS and decodes sweep from the serial port
                        while 1:
                            buffer += self._serial.read(self._serial.inWaiting()).decode() #Constants.app_encoding 
                            #len_buffer = len(buffer)
                            #bar.update(len_buffer)
                            if 's' in buffer:
                                 break
                        #################################################################################
                        # CHANGED v2.0
                        # PRINT LEN BUFFER WHEN THE EOM is RECEIVED
                        #################################################################################    
                        #print("len_buffer = " + str(len_buffer))
                        # bar.finish()
                        
                        # from a full buffer to a list of string
                        data_raw = buffer.split('\n')
                        length = len(data_raw)
                        
                        # PERFORMS split with the semicolon delimiter
                        for i in range (length):
                            strs[i] = data_raw[i].split(';')

                        # converts the sweep samples before adding to queue
                        for i in range (length - 2):
                            data_mag[i] = float(strs[i][0]) * ADCtoVolt / 2
                            data_mag[i] = (data_mag[i]-VCP) / 0.03
                            data_ph[i] = float(strs[i][1]) * ADCtoVolt / 1.5
                            data_ph[i] = (data_ph[i]-VCP) / 0.01
                        
                        #------------------------------
                        if k>0:
                            data_mag=data_mag[1:]
                            data_ph=data_ph[1:]
                        temp1=np.append(temp1,data_mag)
                        temp2=np.append(temp2,data_ph)
                        #print('len=',len(temp1),len(temp2))
                        #------------------------------
                        print(TAG,"signal section #{}/{} acquired successfully\n".format(k+1,Constants.calib_sections), end='\r') #10
                            
                    # specify handlers for different exceptions        
                    except ValueError:
                        print(TAG, "WARNING: ValueError during calibration!")
                        print(TAG, "Please, repeat the calibration after disconnecting/reconnecting device!") 
                        self._flag = 1
                        #Log.w(TAG, "Warning: ValueError during calibration!"))

                        #################################################################################
                        # CHANGED v2.0
                        # SERIAL FLUSH INPUT OUTPUT if an EXCEPTION OCCURR  
                        #################################################################################    
                        self._serial.flushInput()
                        self._serial.flushOutput()
                        self._serial.close()
                        self.stop()

                    except:
                        print(TAG, "WARNING: ValueError during calibration!")
                        print(TAG, "Please, repeat the calibration after disconnecting/reconnecting device!") 
                        self._flag = 1
                        #Log.w(TAG, "Warning (ValueError): convert Raw to float failed") 
                        
                        #################################################################################
                        # CHANGED v2.0
                        # SERIAL FLUSH INPUT OUTPUT if an EXCEPTION OCCURR  
                        #################################################################################    
                        self._serial.flushInput()
                        self._serial.flushOutput()
                        self._serial.close()
                        self.stop()
                    
                    #--------------------------------
                    ## ADDS new serial data to internal queue
                    self._parser1.add1(temp1)
                    self._parser2.add2(temp2)
                    #--------------------------------
                    self._parser6.add6([self._flag,self._flag2,self._flag2,k])
                    k+=1                    
                    # STOPS acquiring data
                    if k==Constants.calib_sections: #10/5
                        self.stop()
                        break
                #### END SWEEPS LOOP ####
                '''
                # CALLS baseline_correction method
                (data_mag_baseline, data_ph_baseline) = self.baseline_correction(readFREQ,temp1,temp2)
                ## ADDS serial data (baseline corrected) to internal queue
                self._parser1.add1(data_mag_baseline)
                self._parser2.add2(data_ph_baseline)
                '''
                #### STORING DATA TO FILE ###
		        # CHECKS QCM Sensor type for saving calibration
                if self._QCStype_int == 0:
                    distance = Constants.dist5
                    path = Constants.cvs_peakfrequencies_path
                    path_calib = Constants.csv_calibration_path
                    filename_calib = Constants.csv_calibration_filename  #
                elif self._QCStype_int == 1:
                    distance = Constants.dist10
                    path = Constants.cvs_peakfrequencies_path
                    path_calib = Constants.csv_calibration_path10
                    filename_calib = Constants.csv_calibration_filename10  #
		        
		        # CHECKS the exceptions
                if self._flag == 0:
                   # CALLS baseline_correction method
                   print(TAG,"Baseline Correction Process Started")
                   (data_mag_baseline, data_ph_baseline) = self.baseline_correction(readFREQ,temp1,temp2)
                   ## ADDS serial data (baseline corrected) to internal queue
                   self._parser1.add1(data_mag_baseline)
                   self._parser2.add2(data_ph_baseline)
                   print(TAG,"Baseline Correction Process Completed")
                   print(TAG,"Peak Detection Process Started")
                   print(TAG, "Finding peaks in acquired signals...")
                   try:
                     # CALLS FindPeak method
                     #(max_freq_mag, max_value_mag, max_freq_phase, max_value_phase)= self.FindPeak(readFREQ, data_mag_baseline, data_ph_baseline, dist=distance)
                     (max_freq_mag, max_value_mag, max_freq_phase, max_value_phase)= self.FindPeak(readFREQ, temp1, temp2, dist=distance)
                     print(TAG, "{} peaks were found at frequencies: {} Hz\n".format(len(max_freq_mag),max_freq_mag))
                     if (len(max_freq_mag)==5 and (max_freq_mag[0]>4e+06 and max_freq_mag[0]<6e+06)) or (len(max_freq_mag)==3 and (max_freq_mag[0]>9e+06 and max_freq_mag[0]<11e+06)):
                        # SAVES independently of the state of the export box
                        print(TAG,"Saving data in file...")
                        np.savetxt(path, np.column_stack([max_freq_mag,max_freq_phase]))
                        print(TAG, "Peak frequencies for {} saved in: {}".format(self._QCStype,path))
                        FileStorage.TXT_sweeps_save(filename_calib, Constants.csv_calibration_export_path, readFREQ, temp1, temp2)
                        print(TAG, "Calibration for {} saved in: {}".format(self._QCStype,path_calib))
                     else:
                        #print('a',max_freq_mag, max_freq_phase)
                        print(TAG, "WARNING: Error during peak detection, incompatible peaks number or frequencies!")
                        print(TAG, "Please, repeat the calibration!")
                        self._flag2 = 1
                   except:
                     #print('b',max_freq_mag, max_freq_phase)
                     print(TAG, "WARNING: Error during peak detection, incompatible peaks number or frequencies!")
                     print(TAG, "Please, repeat the calibration!") 
                     self._flag2 = 1
		             
                if self._flag == 0 and self._flag2 == 0:
                     print(TAG, 'Calibration success for baseline correction!')
                     #print(TAG, 'Please, now click STOP to terminate')
                # ADDS error flags to internal queue
                self._parser5.add5([self._flag,self._flag2])    
                #self._parser6.add6([self._flag,self._flag2,self._flag2,len_buffer])
                #### CLOSES serial port ####
                self._serial.close()
                
          
    ###########################################################################
    # Stops acquiring data
    ###########################################################################
    def stop(self):
        #Signals the process to stop acquiring data.
        self._exit.set()
        
        
    ###########################################################################    
    # Automatically selects the serial ports for Teensy (macox/windows)
    ###########################################################################
    @staticmethod
    def get_ports(): 
        from openQCM.common.architecture import Architecture,OSType
        if Architecture.get_os() is OSType.macosx:
            import glob
            return glob.glob("/dev/tty.usbmodem*") 
        elif Architecture.get_os() is OSType.linux:
            import glob
            return glob.glob("/dev/ttyACM*")
        else:
            found_ports = []
            port_connected = []
            found = False
            ports_avaiable = list(list_ports.comports())
            for port in ports_avaiable:
                if port[2].startswith("USB VID:PID=16C0:0483"):
                    found = True
                    port_connected.append(port[0])
                #else:
                #    Gets a list of the available serial ports.
                #    found_ports.append(port[0])
            if found:
               found_ports = port_connected 
            return found_ports


    ###########################################################################
    # Gets a list of the common serial baud rates, in bps (only 115200 used)
    ###########################################################################
    @staticmethod
    def get_speeds():
        #:return: List of the common baud rates, in bps :rtype: str list.
        return [str(v) for v in ['@10MHz_QCM', '@5MHz_QCM']]#[1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200]]


    ###########################################################################
    # Checks if the serial port is currently connected
    ###########################################################################
    def _is_port_available(self, port):
        """
        :param port: Port name to be verified.
        :return: True if the port is connected to the host :rtype: bool.
        """
        for p in self.get_ports():
            if p == port:
                return True
        return False


# Instantiate the process and run the method 'run' of the class
#a=CalibrationProcess(multiprocessing.Process)
#a.run()




from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Controls(object):
    
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1090, 70)
        MainWindow.setMinimumSize(QtCore.QSize(0, 0))
        MainWindow.setStyleSheet("")
        MainWindow.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.Layout_controls = QtWidgets.QGridLayout()
        self.Layout_controls.setObjectName("Layout_controls")
               
        # frequency/quartz combobox -------------------------------------------
        self.cBox_Speed = QtWidgets.QComboBox(self.centralwidget)
        self.cBox_Speed.setEditable(False)
        self.cBox_Speed.setObjectName("cBox_Speed")
        self.Layout_controls.addWidget(self.cBox_Speed, 4, 1, 1, 1)
        
        # stop button ---------------------------------------------------------
        self.pButton_Stop = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Stop.setObjectName("pButton_Stop")
        self.Layout_controls.addWidget(self.pButton_Stop, 3, 4, 1, 1)
        
        # COM port combobox ---------------------------------------------------
        self.cBox_Port = QtWidgets.QComboBox(self.centralwidget)
        self.cBox_Port.setEditable(False)
        self.cBox_Port.setObjectName("cBox_Port")
        self.Layout_controls.addWidget(self.cBox_Port, 2, 1, 1, 1)
        
        # Operation mode - source ---------------------------------------------
        self.cBox_Source = QtWidgets.QComboBox(self.centralwidget)
        self.cBox_Source.setObjectName("cBox_Source")
        self.Layout_controls.addWidget(self.cBox_Source, 2, 0, 1, 1)
        
        # start button --------------------------------------------------------
        self.pButton_Start = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Start.setIcon(QtGui.QIcon(QtGui.QPixmap("favicon.ico"))) #.png
        self.pButton_Start.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Start.setObjectName("pButton_Start")
        self.Layout_controls.addWidget(self.pButton_Start, 2, 4, 1, 1)
        
        # clear plots button --------------------------------------------------
        self.pButton_Clear = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Clear.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Clear.setObjectName("pButton_Clear")
        self.Layout_controls.addWidget(self.pButton_Clear, 2, 3, 1, 1)
        
        # reference button ----------------------------------------------------
        self.pButton_Reference = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Reference.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Reference.setObjectName("pButton_Reference")
        self.Layout_controls.addWidget(self.pButton_Reference, 3, 3, 1, 1)
        
        # samples SpinBox -----------------------------------------------------
        self.sBox_Samples = QtWidgets.QSpinBox(self.centralwidget)
        self.sBox_Samples.setMinimum(1)
        self.sBox_Samples.setMaximum(100000)
        self.sBox_Samples.setProperty("value", 500)
        self.sBox_Samples.setObjectName("sBox_Samples")
        #self.sBox_Samples.setEnabled(False)
        self.Layout_controls.addWidget(self.sBox_Samples, 2, 2, 1, 1)
        
        # export file CheckBox ------------------------------------------------
        self.chBox_export = QtWidgets.QCheckBox(self.centralwidget)
        self.chBox_export.setEnabled(True)
        self.chBox_export.setObjectName("chBox_export")
        self.Layout_controls.addWidget(self.chBox_export, 4, 2, 1, 1)
        
        # Samples Number / History Buffer Size---------------------------------
        self.l5 = QtWidgets.QLabel()
        self.l5.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l5.setText("<font color=#ffffff > Samples Number / History Buffer Size </font>")
        self.Layout_controls.addWidget(self.l5, 1, 2, 1, 1) 
        
        # Control Buttons------------------------------------------------------
        self.l = QtWidgets.QLabel()
        self.l.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l.setText("<font color=#ffffff > Control Buttons </font>")
        self.Layout_controls.addWidget(self.l, 1, 3, 1, 2)     
        
        # Operation Mode ------------------------------------------------------
        self.l0 = QtWidgets.QLabel()
        self.l0.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l0.setText("<font color=#ffffff >Operation Mode</font> </a>")
        self.Layout_controls.addWidget(self.l0, 1, 0, 1, 1)
        
        # Resonance Frequency / Quartz Sensor ---------------------------------
        self.l2 = QtWidgets.QLabel()
        self.l2.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l2.setText("<font color=#ffffff > Resonance Frequency / Quartz Sensor </font>")
        self.l2.setFixedHeight(15)
        self.Layout_controls.addWidget(self.l2, 3, 1, 1, 1)
        
        # Serial COM Port -----------------------------------------------------
        self.l1 = QtWidgets.QLabel()
        self.l1.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l1.setText("<font color=#ffffff > Serial COM Port </font>")
        self.Layout_controls.addWidget(self.l1, 1, 1, 1, 1)
        
        #######################################################################
        # openQCM logo---------------------------------------------------------
        self.l3 = QtWidgets.QLabel()
        self.l3.setAlignment(QtCore.Qt.AlignRight)
        self.Layout_controls.addWidget(self.l3, 4, 5, 1, 1)
        self.l3.setPixmap(QtGui.QPixmap("openqcm-logo.png"))
        
        # openQCM link --------------------------------------------------------
        self.l4 = QtWidgets.QLabel()
        self.Layout_controls.addWidget(self.l4, 3, 5, 1, 1)
        def link(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.l4.linkActivated.connect(link)
        self.l4.setAlignment(QtCore.Qt.AlignRight)
        self.l4.setText('<a href="https://openqcm.com/openqcm-q-1-software"> <font size=4 color=#008EC0 >openqcm.com</font>') #&nbsp;
                        
        # info@openqcm.com Mail -----------------------------------------------
        self.lmail = QtWidgets.QLabel()
        self.Layout_controls.addWidget(self.lmail, 2, 5, 1, 1)
        def linkmail(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.lmail.linkActivated.connect(linkmail)
        self.lmail.setAlignment(QtCore.Qt.AlignRight)
        self.lmail.setText('<a href="mailto:info@openqcm.com"> <font color=#008EC0 >info@openqcm.com</font>')

        # Save file -----------------------------------------------------------
        self.infosave = QtWidgets.QLabel()
        self.infosave.setStyleSheet('background: #008EC0; padding: 1px;')
        #self.infosave.setAlignment(QtCore.Qt.AlignCenter) 
        self.infosave.setFixedHeight(15)
        self.infosave.setText("<font color=#ffffff > Save file </font>")
        self.Layout_controls.addWidget(self.infosave, 3, 2, 1, 1)
        
        # Program Status standby ----------------------------------------------  
        self.infostatus = QtWidgets.QLabel()
        self.infostatus.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.infostatus.setAlignment(QtCore.Qt.AlignCenter) 
        self.infostatus.setText("<font color=#000000 > Program Status standby </font>")
        self.Layout_controls.addWidget(self.infostatus, 4, 3, 1, 2)
        
        # Infobar -------------------------------------------------------------
        self.infobar = QtWidgets.QLabel()
        self.infobar.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        #self.infobar.setAlignment(QtCore.Qt.AlignCenter) 
        self.infobar.setText("<font color=#0000ff > Infobar </font>")
        self.Layout_controls.addWidget(self.infobar, 0, 0, 1, 5)
        ##
        #self.progressBar = QtWidgets.QProgressBar()
        #self.progressBar.setProperty("value", 0)
        #self.progressBar.setObjectName("progressBar")
        #self.Layout_controls.addWidget(self.progressBar, 5, 2, 1, 2)
        #self.l1a = QtWidgets.QLabel()
        ##
        # ---------------------------------------------------------------------
        
        self.gridLayout.addLayout(self.Layout_controls, 7, 1, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "OPENQCM Q-1 - Setup/Control GUI"))
        MainWindow.setWindowIcon(QtGui.QIcon('favicon.ico')) #.png
        self.pButton_Stop.setText(_translate("MainWindow", "STOP"))
        self.pButton_Start.setText(_translate("MainWindow", "START"))
        self.pButton_Clear.setText(_translate("MainWindow", "Clear Plots"))
        self.pButton_Reference.setText(_translate("MainWindow", "Set/Reset Reference"))
        self.sBox_Samples.setSuffix(_translate("MainWindow", " / 180 min"))
        self.sBox_Samples.setPrefix(_translate("MainWindow", ""))
        self.chBox_export.setText(_translate("MainWindow", "Txt Export Sweep File"))

'''
if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_Controls()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())
'''



import csv
from openQCM.common.fileManager import FileManager
from openQCM.core.constants import Constants
from openQCM.common.logger import Logger as Log
from time import strftime, localtime
import datetime
import numpy as np
import os

TAG = ""#"[FileStorage]"

###############################################################################
# Stores and exports data to file (CSV/TXT): saves incoming and outcoming data
###############################################################################
class FileStorage:
    
    ###########################################################################
    # Saves CSV files of processed data in an assigned directory
    ###########################################################################
    @staticmethod
    def CSVsave(filename, path, data_save0, data_save1, data_save2, data_save3): 
        """
        :param filename: Name for the file :type filename: str.
        :param path: Path for the file     :type path: str.
        :param data_save1: data to store (resonance frequency) :type float. 
        :param data_save2: data to store (dissipation)         :type float.
        """
        # Creates a directory if the specified path doesn't exist
        #FileManager.create_dir(path)
        # Creates a file full path based on parameters
        full_path = FileManager.create_full_path(filename, extension=Constants.csv_extension, path=path)
        if not FileManager.file_exists(full_path):
            print("\n")
            print(TAG, "Exporting data to CSV file...")
            print(TAG, "Storing in: {}".format(full_path))
            Log.i(TAG, "Storing in: {}".format(full_path))
            
        # checks the path for the header insertion
        if os.path.exists(full_path):
           header_exists = True
        else:
           header_exists = False
           
        # opens the file to write data
        with open(full_path,'a', newline='') as tempFile:   
         tempFileWriter = csv.writer(tempFile)
         # inserts the header if it doesn't exist
         if not header_exists:
            tempFileWriter.writerow(["Date","Time","Relative_time","Temperature", "Resonance_Frequency", "Dissipation"])
         #csv_time_prefix = strftime(Constants.csv_default_prefix, localtime())
         #now = datetime.datetime.now()
         #csv_time_prefix = "{}-{}-{}.{}:{}:{}.{}".format(now.year,now.month,now.day,now.hour,now.minute,now.second,now.microsecond)         
         fix1= "%Y-%m-%d"
         fix2= "%H:%M:%S"
         csv_time_prefix1 = (strftime(fix1, localtime()))
         csv_time_prefix2 = (strftime(fix2, localtime()))
         d0=float("{0:.2f}".format(data_save0))
         d1=float("{0:.2f}".format(data_save1))
         d2=float("{0:.2f}".format(data_save2))
         #d3=float("{0:.4e}".format(data_save3))
         tempFileWriter.writerow([csv_time_prefix1,csv_time_prefix2, d0, d1, d2, data_save3])
         #tempFileWriter.writerow([csv_time_prefix, data_save1, data_save2, data_save3])
        tempFile.close()
    
    
    ###########################################################################
    # Saves a CSV-formatted CSV file per sweeps in an assigned directory
    ###########################################################################     
    @staticmethod
    def CSV_sweeps_save(filename, path, data_save1, data_save2, data_save3):
        """
        :param filename: Name for the file :type filename: str.
        :param path: Path for the file     :type path: str.
        :param data_save1: data to store (frequency) :type float. 
        :param data_save2: data to store (Amplitude) :type float.
        :param data_save2: data to store (Phase)     :type float.
        """
        # Creates a directory if the specified path doesn't exist
        FileManager.create_dir(path)
        # Creates a file full path based on parameters
        full_path = FileManager.create_full_path(filename, extension=Constants.csv_extension, path=path)
        # creates CSV file
        np.savetxt(full_path, np.column_stack([data_save1, data_save2, data_save3]), delimiter=',')
    
    
    ###########################################################################
    # Saves a CSV-formatted Text file per sweeps in an assigned directory
    ###########################################################################     
    @staticmethod
    def TXT_sweeps_save(filename, path, data_save1, data_save2, data_save3):
        """
        :param filename: Name for the file :type filename: str.
        :param path: Path for the file     :type path: str.
        :param path: Path for the file     :type path: str.
        :param data_save1: data to store (frequency) :type float. 
        :param data_save2: data to store (Amplitude) :type float.
        :param data_save2: data to store (Phase)     :type float.
        """
        # Creates a directory if the specified path doesn't exist
        FileManager.create_dir(path)
        # Creates a file full path based on parameters
        full_path = FileManager.create_full_path(filename, extension=Constants.txt_extension, path=path)
        # creates TXT file
        np.savetxt(full_path, np.column_stack([data_save1, data_save2, data_save3]))  



import multiprocessing
from time import time, sleep

import numpy as np

from openQCM.core.constants import Constants
from openQCM.common.logger import Logger as Log


TAG = "Simulator"


class SimulatorProcess(multiprocessing.Process):
    """
    Simulates signals and converts them as raw data to feed the processes.
    """
    def __init__(self, parser_process):
        """
        Initialises values for process.
        :param parser_process: Reference to a ParserProcess instance.
        :type parser_process: ParserProcess.
        """
        multiprocessing.Process.__init__(self)
        self._exit = multiprocessing.Event()
        self._period = None
        self._parser = parser_process
        Log.i(TAG, "Process Ready")

    def open(self, port=None, speed=Constants.simulator_default_speed, timeout=0.5):
        """
        Opens a specified serial port.
        :param port: Not used.
        :type port: str.
        :param speed: Period of the generated signal.
        :type speed: float.
        :param timeout: Not used.
        :type timeout: float.
        :return: True if the port is available.
        :rtype: bool.
        """
        self._period = float(speed)
        Log.i(TAG, "Using sample rate at {}".format(self._period))
        return True

    def run(self):
        """
        Simulates raw data incoming as CSV.
        :return:
        """
        Log.i(TAG, "Process starting...")
        timestamp = time()
        coef = 2 * np.pi
        while not self._exit.is_set():
            stamp = time() - timestamp
            self._parser.add([stamp, str(("{},{},{}\r\n".format(np.sin(coef * stamp), np.cos(coef * stamp), np.cos(1.5*coef * stamp))))
                             .encode(Constants.app_encoding)])
            sleep(self._period)
        Log.i(TAG, "Process finished")

    def stop(self):
        """
        Signals the process to stop acquiring data.
        :return:
        """
        Log.i(TAG, "Process finishing...")
        self._exit.set()

    @staticmethod
    def get_ports():
        """
        Gets a list of the available ports.
        :return: List of available ports.
        :rtype: str list.
        """
        return ["Sine Simulator"]

    @staticmethod
    def get_speeds():
        """
        Gets a list of the speeds.
        :return: List of the speeds.
        :rtype: str list.
        """
        return [str(v) for v in [0.002, 0.004, 0.005, 0.010, 0.020, 0.050, 0.100, 0.250]]



from PyQt5 import QtGui

TAG = "[PopUp]"

###############################################################################
# Warning dialog module
###############################################################################
class PopUp:
    
    ###########################################################################
    # Shows a pop-up question dialog with yes and no buttons (unused)
    ###########################################################################
    @staticmethod
    def question_QCM(parent, title, message):
        """
        :param parent: Parent window for the dialog.
        :param title: Title of the dialog :type title: str.
        :param message: Message to be shown in the dialog :type message: str.
        :return: 1 if button1 was pressed, 0 if button2   :rtype: int.
        """
        #ans = QtGui.QMessageBox.question(parent, title, message, QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
        #if ans == QtGui.QMessageBox.Yes:
        #    print('Si')
        #    return True
        #elif ans == QtGui.QMessageBox.No:
        #    print('No')
        #    return False
        left = 700
        top = 400
        width = 340
        height = 220
        box = QtGui.QMessageBox(parent)
        box.setIcon(QtGui.QMessageBox.Question)
        box.setWindowTitle(title)
        box.setGeometry(left, top, width, height)
        box.setText(message)
        box.setStandardButtons(QtGui.QMessageBox.Yes|QtGui.QMessageBox.No)
        button1 = box.button(QtGui.QMessageBox.Yes)
        button1.setText('@10MHz')
        button2 = box.button(QtGui.QMessageBox.No)
        button2.setText(' @5MHz')
        box.exec_()
        
        if box.clickedButton() == button1:
            print(TAG, 'Quartz Crystal Sensor installed on the openQCM Device: @10MHz')
            return 1
        elif box.clickedButton() == button2:
            print(TAG, 'Quartz Crystal Sensor installed on the openQCM Device: @5MHz')
            return 0

    ###########################################################################
    # Shows a Pop up warning dialog with a Ok buttons
    ###########################################################################
    @staticmethod
    def warning(parent, title, message):
        """
        :param parent: Parent window for the dialog.
        :param title: Title of the dialog :type title: str.
        :param message: Message to be shown in the dialog :type message: str.
        """
        QtGui.QMessageBox.warning(parent, title, message, QtGui.QMessageBox.Ok)
        #msgBox=QtGui.QMessageBox.warning(parent, title, message, QtGui.QMessageBox.Ok)
        #msgBox = QtGui.QMessageBox()
        #msgBox.setIconPixmap( QtGui.QPixmap("favicon.png"))
        #msgBox.exec_() 

    ###########################################################################
    # Shows a pop-up question dialog with yes and no buttons
    ###########################################################################
    @staticmethod
    def question(parent, title, message):
        """
        :param parent: Parent window for the dialog.
        :param title: Title of the dialog :type title: str.
        :param message: Message to be shown in the dialog :type message: str.
        :return: True if Yes button was pressed :rtype: bool.
        """
        ans = QtGui.QMessageBox.question(parent, title, message, QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
        if ans == QtGui.QMessageBox.Yes:
            return True
        else:
            return False




from PyQt5 import QtCore, QtGui, QtWidgets
from pyqtgraph import GraphicsLayoutWidget

class Ui_Plots(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1091, 770)
        MainWindow.setMinimumSize(QtCore.QSize(0, 0))
        MainWindow.setStyleSheet("")
        MainWindow.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.Layout_graphs = QtWidgets.QGridLayout()
        self.Layout_graphs.setObjectName("Layout_graphs")
        
        self.plt = GraphicsLayoutWidget(self.centralwidget)
        self.pltB = GraphicsLayoutWidget(self.centralwidget)
        
        self.plt.setAutoFillBackground(False)
        self.plt.setStyleSheet("border: 0px;")
        self.plt.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.plt.setFrameShadow(QtWidgets.QFrame.Plain)
        self.plt.setLineWidth(0)
        self.plt.setObjectName("plt")
        
        self.pltB.setAutoFillBackground(False)
        self.pltB.setStyleSheet("border: 0px;")
        self.pltB.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.pltB.setFrameShadow(QtWidgets.QFrame.Plain)
        self.pltB.setLineWidth(0)
        self.pltB.setObjectName("pltB")
        
        self.label = QtWidgets.QLabel()
        self.Layout_graphs.addWidget(self.label, 0, 0, 1, 1)
        def link1(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.label.linkActivated.connect(link1)
        self.label.setText('<a href="https://openqcm.com/"> <font color=#000000 >Open-source Python application for displaying, processing and storing real-time data from openQCM Q-1 Device</font> </a>')
        
        self.Layout_graphs.addWidget(self.plt, 1, 0, 1, 1)
        self.Layout_graphs.addWidget(self.pltB, 2, 0, 1, 1)
        
        self.gridLayout.addLayout(self.Layout_graphs, 3, 1, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowIcon(QtGui.QIcon('favicon.ico')) #.png
        MainWindow.setWindowTitle(_translate("MainWindow", "OPENQCM Q-1 - Setup/Control GUI"))

'''
if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_Plots()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())
'''



from enum import Enum
import numpy as np
from pyqtgraph import AxisItem
from time import strftime, localtime
import time
import datetime
import os

import openQCM
from openQCM.common.architecture import Architecture,OSType

###############################################################################    
# Enum for the types of sources. Indices MUST match app_sources constant
###############################################################################
class SourceType(Enum):
    serial = 0
    calibration = 1
    SocketClient = 2
    

###############################################################################
# Specifies the minimal Python version required
###############################################################################
class MinimalPython:
    major = 3
    minor = 2
    release = 0    
 
  
###############################################################################    
# Common constants and parameters for the application.
###############################################################################
class Constants:
    
    ##########################
    # Application Parameters #
    ##########################
    app_title = "Real-Time openQCM GUI"
    app_version = openQCM.__version__
    app_sources = ["Measurement openQCM Q-1 Device", "Calibration openQCM Q-1 Device"]#, "Socket Client"]
    app_encoding = "utf-8"
    
    
    ###################
    # PLOT parameters #
    ###################
    plot_update_ms = 200#16
    plot_colors = ['#ff0000', '#0072bd', '#00ffff', '#edb120', '#7e2f8e', '#77ac30', '#4dbeee', '#a2142f'] 
    plot_max_lines = len(plot_colors)
    
    
    ####################
    #  SAMPLES NUMBER  #
    ####################
    argument_default_samples = 501#1001
    
    
    ####################################
    # FILTERING and FITTING parameters #
    ####################################
    # Notes:
    # left and right frequencies in the area of the resonance frequency
    # Savitzky-Golay size of the data window 
    # Savitzky-Golay order of the polynomial fit
    # Number of spline points: same as the frequency band +1 (es.5001)
    # Spline smoothing factor
    
    # Savitzky-Golay order of the polynomial fit (common for all)
    SG_order = 3

    #--------------
    # 5MHz 
    #--------------
    # left and right frequencies
    L5_fundamental = 5500
    R5_fundamental = 2500
    # Savitzky-Golay size of the data window 
    SG_window_size5_fundamental = 9
    # Spline smoothing factor
    Spline_factor5_fundamental = 0.05
    # left and right frequencies
    L5_3th_overtone = 7500
    R5_3th_overtone = 2500
    # Savitzky-Golay size of the data window 
    SG_window_size5_3th_overtone = 11
    # Spline smoothing factor
    Spline_factor5_3th_overtone = 0.01
    
    # left and right frequencies
    L5_5th_overtone = 10000
    R5_5th_overtone = 2500
    # Savitzky-Golay size of the data window 
    SG_window_size5_5th_overtone = 11
    # Spline smoothing factor
    Spline_factor5_5th_overtone = 0.01
    
    # left and right frequencies
    L5_7th_overtone = 50000
    R5_7th_overtone = 2500
    # Savitzky-Golay size of the data window 
    SG_window_size5_7th_overtone = 33
    # Spline smoothing factor
    Spline_factor5_7th_overtone = 0.01
    
    # TODO
    # left and right frequencies 
    L5_9th_overtone = 5000000
    R5_9th_overtone = 100000
    # Savitzky-Golay size of the data window 
    SG_window_size5_9th_overtone = 5
    # Spline smoothing factor
    Spline_factor5_9th_overtone = 0.5
    
    #--------------
    # 10MHz 
    #--------------
    # left and right frequencies
    L10_fundamental = 7500
    R10_fundamental = 2500
    # Savitzky-Golay size of the data window 
    SG_window_size10_fundamental = 11
    # Spline smoothing factor
    Spline_factor10_fundamental = 0.01
    
    # left and right frequencies
    L10_3th_overtone = 13500
    R10_3th_overtone = 2500
    # Savitzky-Golay size of the data window 
    SG_window_size10_3th_overtone = 11    
    # Spline smoothing factor
    Spline_factor10_3th_overtone = 0.01
    
    # left and right frequencies
    L10_5th_overtone = 23000
    R10_5th_overtone = 3000
    # Savitzky-Golay size of the data window 
    SG_window_size10_5th_overtone = 19
    # Spline smoothing factor
    Spline_factor10_5th_overtone = 0.01


    ##########################
    # SERIAL PORT Parameters #
    ##########################
    serial_default_speed = 115200
    serial_default_overtone = None
    serial_default_QCS = "@10MHz"
    serial_writetimeout_ms = 0
    serial_timeout_ms = None#0.01

    
    ######################
    # Process parameters #
    ######################
    process_join_timeout_ms = 2000
    simulator_default_speed = 0.1 # not used
    parser_timeout_ms = 0.005
    
    
    ##################
    # Log parameters #
    ##################
    log_export_path = "logged_data"
    log_filename = "{}.log".format(app_title)
    log_max_bytes = 5120
    log_default_level = 1
    log_default_console_log = False
    

    ######################################
    # File parameters for exporting data #
    ######################################
    # sets the slash depending on the OS types
    if Architecture.get_os() in [OSType.macosx, OSType.linux]:
       slash="/"
    else:
       slash="\\"
       
    csv_delimiter = "," # for splitting data of the serial port and CSV file storage
    csv_default_prefix = "%Y-%b-%d_%H-%M-%S"#"%H-%M-%S-%d-%b-%Y" # Hour-Minute-Second-month-day-Year
    csv_extension = "csv"
    txt_extension = "txt"
    csv_export_path = "logged_data"
    csv_filename = (strftime(csv_default_prefix, localtime()))#+'_DataLog')
    csv_sweeps_export_path = "{}{}{}".format(csv_export_path,slash,csv_filename)
    csv_sweeps_filename = "sweep"
    
    # Calibration: scan (WRITE for @5MHz and @10MHz QCS) path: 'common\'
    csv_calibration_filename    = "Calibration_5MHz"
    csv_calibration_filename10  = "Calibration_10MHz"
    csv_calibration_export_path = os.path.dirname(openQCM.__file__) #"common"
    
    ################## 
    # Calibration: baseline correction (READ for @5MHz and @10MHz QCS) path: 'common\'
    csv_calibration_path   = "{}{}{}.{}".format(csv_calibration_export_path,slash,csv_calibration_filename,txt_extension)
    csv_calibration_path10 = "{}{}{}.{}".format(csv_calibration_export_path,slash,csv_calibration_filename10,txt_extension)
    
    # Frequencies: Fundamental and overtones (READ and WRITE for @5MHz and @10MHz QCS)
    csv_peakfrequencies_filename   = "PeakFrequencies"
    #csv_peakfrequencies_filename   = "PeakFrequencies_5MHz"
    #csv_peakfrequencies_filename10 = "PeakFrequencies_10MHz"
    cvs_peakfrequencies_path    = "{}{}{}.{}".format(csv_calibration_export_path,slash,csv_peakfrequencies_filename,txt_extension)
    #cvs_peakfrequencies_path10 = "{}{}{}.{}".format(csv_calibration_export_path,slash,csv_peakfrequencies_filename10,txt_extension)    
    #########################    
    '''
    # Calibration: baseline correction (READ for @5MHz and @10MHz QCS) path: 'common\'
    csv_calibration_path   = "{}\{}.{}".format(csv_calibration_export_path,csv_calibration_filename,txt_extension)
    csv_calibration_path10 = "{}\{}.{}".format(csv_calibration_export_path,csv_calibration_filename10,txt_extension)
    
    # Frequencies: Fundamental and overtones (READ and WRITE for @5MHz and @10MHz QCS)
    csv_peakfrequencies_filename   = "PeakFrequencies"
    #csv_peakfrequencies_filename   = "PeakFrequencies_5MHz"
    #csv_peakfrequencies_filename10 = "PeakFrequencies_10MHz"
    cvs_peakfrequencies_path    = "{}\{}.{}".format(csv_calibration_export_path,csv_peakfrequencies_filename,txt_extension)
    #cvs_peakfrequencies_path10 = "{}\{}.{}".format(csv_calibration_export_path,csv_peakfrequencies_filename10,txt_extension)
    '''
    
    ##########################
    # CALIBRATION PARAMETERS #
    ##########################
    
    # Peak Detection - distance in samples between neighbouring peaks
    dist5  =  8000 # for @5MHz
    dist10 =  10000 # for @10MHz
    calibration_default_samples = 50001
    calibration_frequency_start =  1000000
    calibration_frequency_stop  = 51000000 
    calibration_fStep = (calibration_frequency_stop - calibration_frequency_start) / (calibration_default_samples-1)
    calibration_readFREQ  = np.arange(calibration_default_samples) * (calibration_fStep) + calibration_frequency_start
    #-------------------
    calib_fStep = 1000
    calib_fRange = 5000000 #
    calib_samples = 5001
    calib_sections = 10
    
    ###########################
    # Ring Buffers Parameters #
    ###########################
    ring_buffer_samples = 16363
    
    ##############################
    # Parameters for the average #
    ##############################  
    environment = 50 #######################################################################################################
    SG_order_environment = 1
    SG_window_environment = 3
    
    ###################
    class SocketClient: #unused
        timeout = 0.01
        host_default = "localhost"
        port_default = [5555, 8080, 9090]
        buffer_recv_size = 1024
    ###################  



'''
###############################################################################
#  Provides a date-time aware axis
###############################################################################    
class DateAxis(AxisItem):
    
    """
    A tool that provides a date-time aware axis. It is implemented as an AxisItem 
    that interprets positions as UNIX timestamps (i.e. seconds since 1970). 
    The labels and the tick positions are dynamically adjusted depending on the range.
    """

    def __init__(self, *args, **kwargs):
        AxisItem.__init__(self, *args, **kwargs)
        self._oldAxis = None
    
    def tickStrings(self, values, scale, spacing):
        ret = []
        ep = datetime.datetime(1970,1,1,0,0,0)
        tonow = (datetime.datetime.utcnow()- ep).total_seconds()
        if not values:
            return []
        if spacing >= 31622400:  #366days
            fmt = "%Y"
        elif spacing >= 2678400: #31days
            fmt = "%Y %b"
        elif spacing >= 86400:   #1day
            fmt = "%b/%d"
        elif spacing >= 3600:    #1h
            fmt = "%b/%d-%Hh"
        elif spacing >= 60:      #1m
            fmt = "%H:%M"
        elif spacing >= 1:       #1s
            fmt = "%H:%M:%S"
        else: # less than 2s (show microseconds)
            #fmt = "%S.%f"""
            fmt = '[+%fms]'  # explicitly relative to last second   
        for x in values:
            try:
                ret.append(time.strftime(fmt, time.localtime(x*.1+tonow))) #time.localtime(x*.1+tonow)
            except ValueError:  # Windows can't handle dates before 1970
                ret.append('')
            except:
                ret.append('')    
        return ret
'''
###############################################################################
#  Provides a date-time aware axis
###############################################################################  
class DateAxis(AxisItem): 
    def __init__(self, *args, **kwargs):
        super(DateAxis, self).__init__(*args, **kwargs)            
          
    def tickStrings(self, values, scale, spacing):    
        TS_MULT_us = 1e6
        try:
            z= [(datetime.datetime.utcfromtimestamp(float(value)/TS_MULT_us)).strftime("%b-%d %H:%M:%S") for value in values]
        except: 
            z= ''
        return z
        #return [(datetime.datetime.utcfromtimestamp(float(value)/TS_MULT_us)).strftime("%b-%d %H:%M:%S") for value in values]


###############################################################################
#  Provides a non scientific axis notation
###############################################################################  
class NonScientificAxis(AxisItem):
    def __init__(self, *args, **kwargs):
        super(NonScientificAxis, self).__init__(*args, **kwargs)

    def tickStrings(self, values, scale, spacing):
        return [int(value*1) for value in values] 



import os
from openQCM.common.architecture import Architecture,OSType

###############################################################################
# File operations: create directory, full path and check if the existing file
###############################################################################
class FileManager:

    ###########################################################################
    # Creates a directory if the specified path doesn't exist.
    ###########################################################################
    @staticmethod
    def create_dir(path=None):
        """
        :param path: Directory name or full path        :type path: str.
        :return: True if the specified directory exists :rtype: bool.
        """
        if path is not None:
            if not os.path.isdir(path):
                os.makedirs(path)
        return os.path.isdir(path)

    
    ###########################################################################
    # Creates a file full path based on parameters
    ###########################################################################    
    @staticmethod
    def create_full_path(filename, extension="txt", path=None):
        """
        :param filename: Name for the file        :type filename: str.
        :param extension: Extension for the file  :type extension: str.
        :param path: Path for the file, if needed :type path: str.
        :return: Full path for the specified file :rtype: str.
        """
        # sets the slash depending on the OS types
        if Architecture.get_os() is (OSType.macosx or OSType.linux):
            slash="/"
        else:
            slash="\\"
            
        if path is None:
            full_path = str("{}.{}".format(filename, extension))
        else:
            full_path = str("{}{}{}.{}".format(path,slash, filename, extension))
        return full_path


    ###########################################################################
    #Checks if a file exists (True if file exists)
    ###########################################################################
    @staticmethod
    def file_exists(filename):
        """
        :param filename: Name of the file, including path :type filename: str.
        :return: True if file exists :rtype: bool.
        """
        if filename is not None:
            return os.path.isfile(filename)



import logging
import logging.handlers
import sys
from enum import Enum

from openQCM.common.architecture import Architecture
from openQCM.common.fileManager import FileManager
from openQCM.core.constants import Constants

###############################################################################
# Logging package - All packages can use this module
###############################################################################
class Logger:

    ###########################################################################
    # Creates logging file (.txt)
    ###########################################################################
    def __init__(self, level, enable_console=True):
        """
        :param level: Level to show in log.
        :type level: int.
        """
        log_format_file = logging.Formatter('%(asctime)s,%(levelname)s,%(message)s')
        log_format_console = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        self.logger = logging.getLogger()
        self.logger.setLevel(level.value)

        FileManager.create_dir(Constants.log_export_path)
        file_handler = logging.handlers.RotatingFileHandler("{}/{}"
                                                            .format(Constants.log_export_path, Constants.log_filename),
                                                            maxBytes=Constants.log_max_bytes,
                                                            backupCount=0)
        file_handler.setFormatter(log_format_file)
        self.logger.addHandler(file_handler)

        if enable_console:
            console_handler = logging.StreamHandler(sys.stdout)
            console_handler.setFormatter(log_format_console)
            self.logger.addHandler(console_handler)
        self._show_user_info()

    ###########################################################################
    # Closes the enabled loggers.
    ###########################################################################
    @staticmethod
    def close():
        logging.shutdown()

    ###########################################################################
    # Logs at debug level (debug,info,warning and error messages)
    ###########################################################################
    @staticmethod
    def d(tag, msg):
        """
        :param tag: TAG to identify the log :type tag: str.
        :param msg: Message to log.         :type msg: str.
        """
        logging.debug("[{}] {}".format(str(tag), str(msg)))
    
    ####
    @staticmethod
    def i(tag, msg):
        logging.info("[{}] {}".format(str(tag), str(msg)))
    
    ####
    @staticmethod
    def w(tag, msg):
        logging.warning("[{}] {}".format(str(tag), str(msg)))
    
    ####
    @staticmethod
    def e(tag, msg):
        logging.error("[{}] {}".format(str(tag), str(msg)))


    ###########################################################################
    # logs and prints architecture-related informations
    ###########################################################################
    @staticmethod
    def _show_user_info():
        tag = ""#"[User]"
        print("-----------------------------")
        print(" {} - {}".format(Constants.app_title,Constants.app_version))
        print("-----------------------------")
        print("\n{} SYSTEM INFORMATIONS:".format(tag))
        print(tag,"Platform: {}".format(Architecture.get_os_name()))
        Logger.i(tag, "Platform: {}".format(Architecture.get_os_name()))
        Logger.i(tag, "Path: {}".format(Architecture.get_path()))
        print(tag,"Python version: {}".format(Architecture.get_python_version()))
        Logger.i(tag, "Python version: {}".format(Architecture.get_python_version()))


###############################################################################
# Enumeration for the Logger levels
###############################################################################        
class LoggerLevel(Enum):
    CRITICAL = logging.CRITICAL
    ERROR = logging.ERROR
    WARNING = logging.WARNING
    INFO = logging.INFO
    DEBUG = logging.DEBUG




from openQCM.ui.mainWindow_ui import Ui_Controls, Ui_Info, Ui_Plots
#from openQCM.ui.ui_controls import Ui_Controls
#from openQCM.ui.ui_info import Ui_Info
#from openQCM.ui.ui_plots import Ui_Plots

from pyqtgraph import AxisItem
import pyqtgraph as pg
from PyQt5 import QtCore, QtGui
from openQCM.core.worker import Worker
from openQCM.core.constants import Constants, SourceType, DateAxis, NonScientificAxis
from openQCM.ui.popUp import PopUp
from openQCM.common.logger import Logger as Log
import numpy as np
import sys

TAG = ""#"[MainWindow]"

##########################################################################################
# Package that handles the UIs elements and connects to worker service to execute processes
##########################################################################################

class PlotsWindow(QtGui.QMainWindow):
     
    def __init__(self, samples=Constants.argument_default_samples):
        super().__init__()  
        self.ui2 = Ui_Plots()
        self.ui2.setupUi(self)
    '''
    def closeEvent(self, event):
        #print(" Exit Real-Time Plot GUI")
        res =PopUp.question(self, Constants.app_title, "Are you sure you want to quit openQCM application now?")
        if res: 
           #self.close()
           QtGui.QApplication.quit()
        else:
           event.ignore()#pass
        #sys.exit(0)
    '''
    def closeEvent(self, event):
        #print(" Exit Real-Time Plot GUI")
           event.ignore()

    
#------------------------------------------------------------------------------
class InfoWindow(QtGui.QMainWindow):    
    
    def __init__(self, samples=Constants.argument_default_samples):
        super().__init__()
        self.ui3 = Ui_Info()
        self.ui3.setupUi(self)
    '''    
    def closeEvent(self, event):
        #print(" Exit Information GUI") 
        res =PopUp.question(self, Constants.app_title, "Are you sure you want to quit openQCM application now?")
        if res: 
           #self.close()
           QtGui.QApplication.quit()
        else:
           event.ignore()
    '''
    def closeEvent(self, event):
        #print(" Exit Real-Time Plot GUI")
           event.ignore()    

#------------------------------------------------------------------------------
class ControlsWindow(QtGui.QMainWindow):    
    
    def __init__(self, samples=Constants.argument_default_samples):
        super().__init__()
        self.ui1 = Ui_Controls()
        self.ui1.setupUi(self)
        
    def closeEvent(self, event):
        #print(" Exit Setup/Control GUI")
        res =PopUp.question(self, Constants.app_title, "Are you sure you want to quit openQCM application now?")
        if res: 
           #self.close()
           QtGui.QApplication.quit()
        else:
           event.ignore()

#------------------------------------------------------------------------------

class MainWindow(QtGui.QMainWindow):
    
    ###########################################################################
    # Initializes methods, values and sets the UI
    ###########################################################################   
    def __init__(self, samples=Constants.argument_default_samples):
        
        #:param samples: Default samples shown in the plot :type samples: int.
        # to be always placed at the beginning, initializes some important methods
        QtGui.QMainWindow.__init__(self)
        
        # Sets up the user interface from the generated Python script using Qt Designer
        # Instantiates Ui classes

        # Calls setupUi method of created instance
        self.ControlsWin = ControlsWindow()
        self.ControlsWin.move(530, 568)
        self.PlotsWin = PlotsWindow()
        self.PlotsWin.move(530,10) #GUI position (x,y) on the screen 
        self.InfoWin = InfoWindow()
        self.InfoWin.move(1242, 10)
        self.ControlsWin.show()
        self.PlotsWin.show()
        self.InfoWin.show()

        # Shared variables, initial values
        self._plt0 = None
        self._plt1 = None
        self._plt2 = None
        self._plt3 = None
        self._plt4 = None
        self._timer_plot = None
        self._readFREQ = None
        self._QCS_installed = None
        self._ser_control = 0
        self._ser_error1 = 0
        self._ser_error2 = 0
        self._ser_err_usb= 0
        
        # internet connection variable
        self._internet_connected = False
        
        # Reference variables
        self._reference_flag = False 
        self._vector_reference_frequency = None
        self._vector_reference_dissipation = None
        self._vector_1 = None
        self._vector_2 = None   
        
        # Instantiates a Worker class 
        self.worker = Worker()

        # Populates comboBox for sources
        self.ControlsWin.ui1.cBox_Source.addItems(Constants.app_sources)

        # Configures specific elements of the PyQtGraph plots
        self._configure_plot()
       
        # Configures specific elements of the QTimers
        self._configure_timers()
        
        # Configures the connections between signals and UI elements
        self._configure_signals()

        # Populates combo box for serial ports
        self._source_changed()
        self.ControlsWin.ui1.cBox_Source.setCurrentIndex(SourceType.serial.value)
        self.ControlsWin.ui1.sBox_Samples.setValue(samples)  #samples
        
        # enable ui
        self._enable_ui(True)
        ###################################################################################################################################
        self.get_web_info()
        # Gets the QCS installed on the device (not used now)
        # self._QCS_installed = PopUp.question_QCM(self, Constants.app_title, "Please choose the Quartz Crystal Resonator installed on the openQCM-1 Device (default 5MHz if exit)")

    ###########################################################################
    # Starts the acquisition of the selected serial port
    ###########################################################################      
    def start(self):
        
        import os
        os.system('cls' if os.name == 'nt' else 'clear')
        
        # This function is connected to the clicked signal of the Start button.
        #print("")
        print(TAG, 'Clicked START')
        Log.i(TAG, "Clicked START")
        # Instantiates process
        self.worker = Worker(QCS_on = self._QCS_installed,
                             port = self.ControlsWin.ui1.cBox_Port.currentText(),
                             speed = self.ControlsWin.ui1.cBox_Speed.currentText(),
                             samples = self.ControlsWin.ui1.sBox_Samples.value(),
                             source = self._get_source(),
                             export_enabled = self.ControlsWin.ui1.chBox_export.isChecked())     
        
        if self.worker.start():
            # Gets frequency range 
            self._readFREQ = self.worker.get_frequency_range()
            # Duplicate frequencies
            self._reference_flag = False
            self._vector_reference_frequency = list(self._readFREQ)
            self._reference_value_frequency = 0
            self._reference_value_dissipation = 0
            self._labelref1 = "not set"
            self._labelref2 = "not set"
            # progressbar variables
            self._completed=0
            self._ser_control = 0
            # error variables
            self._ser_error1 = 0
            self._ser_error2 = 0
            self._ser_err_usb= 0
            ##### other useful location #########
            #self.get_web_info()
            #####

            if self._get_source() == SourceType.serial:
                overtones_number = len(self.worker.get_source_speeds(SourceType.serial))
                if overtones_number==5:
                   label_quartz = "@5MHz_QCM"
                elif overtones_number==3:
                   label_quartz = "@10MHz_QCM"
                self.InfoWin.ui3.info1a.setText("<font color=#0000ff > Device Setup </font>" + label_quartz)
                label11= "Measurement openQCM Q-1"
                self.InfoWin.ui3.info11.setText("<font color=#0000ff > Operation Mode </font>" + label11)
                self._overtone_name,self._overtone_value, self._fStep = self.worker.get_overtone()
                label6= str(int(self._overtone_value))+" Hz"
                self.InfoWin.ui3.info6.setText("<font color=#0000ff > Frequency Value </font>" + label6)
                label2= str(self._overtone_name)# +" "+ str(self._overtone_value)+"Hz"
                self.InfoWin.ui3.info2.setText("<font color=#0000ff > Selected Frequency </font>" + label2)
                label3= str(int(self._readFREQ[0]))+" Hz"
                self.InfoWin.ui3.info3.setText("<font color=#0000ff > Start Frequency </font>" + label3)
                label4= str(int(self._readFREQ[-1]))+" Hz"
                self.InfoWin.ui3.info4.setText("<font color=#0000ff > Stop Frequency </font>" + label4)
                label4a= str(int(self._readFREQ[-1]-self._readFREQ[0]))+" Hz"
                self.InfoWin.ui3.info4a.setText("<font color=#0000ff > Frequency Range </font>" + label4a)                      
                label5= str(int(self._fStep))+" Hz"
                self.InfoWin.ui3.info5.setText("<font color=#0000ff > Sample Rate </font>" + label5) 
                label7= str(Constants.argument_default_samples-1)
                self.InfoWin.ui3.info7.setText("<font color=#0000ff > Sample Number </font>" + label7)
                                     
            elif self._get_source() == SourceType.calibration:
                label_quartz = self.ControlsWin.ui1.cBox_Speed.currentText()
                self.InfoWin.ui3.info1a.setText("<font color=#0000ff > Device Setup </font>" + label_quartz)
                label11= "Calibration openQCM Q-1"
                self.InfoWin.ui3.info11.setText("<font color=#0000ff > Operation Mode </font>" + label11)
                label6= "Overall Frequency Range"
                self.InfoWin.ui3.info6.setText("<font color=#0000ff > Frequency Value </font>" + label6)
                label2= "Overall Frequency Range"
                self.InfoWin.ui3.info2.setText("<font color=#0000ff > Selected Frequency </font>" + label2)
                label3= str(Constants.calibration_frequency_start)+" Hz"
                self.InfoWin.ui3.info3.setText("<font color=#0000ff > Start Frequency </font>" + label3)
                label4= str(Constants.calibration_frequency_stop)+" Hz"
                self.InfoWin.ui3.info4.setText("<font color=#0000ff > Stop Frequency </font>" + label4) 
                label4a= str(int(Constants.calibration_frequency_stop - Constants.calibration_frequency_start))+" Hz"
                self.InfoWin.ui3.info4a.setText("<font color=#0000ff > Frequency Range </font>" + label4a)                        
                label5= str(int(Constants.calibration_fStep))+" Hz"
                self.InfoWin.ui3.info5.setText("<font color=#0000ff > Sample Rate </font>" + label5)  
                label7= str(Constants.calibration_default_samples-1)
                self.InfoWin.ui3.info7.setText("<font color=#0000ff > Sample Number </font>" + label7)  
            #
            self._timer_plot.start(Constants.plot_update_ms)
            self._timer_plot.timeout.connect(self._update_plot) #moved from _configure_timers mothod
            self._enable_ui(False)
            self.ControlsWin.ui1.sBox_Samples.setEnabled(False) #insert
            
            if self._get_source() == SourceType.calibration:
               self.ControlsWin.ui1.pButton_Clear.setEnabled(False) #insert
               self.ControlsWin.ui1.pButton_Reference.setEnabled(False) #insert
        else:
            print(TAG, "Warning: port is not available!")
            Log.i(TAG, "Warning: port is not available")
            PopUp.warning(self, Constants.app_title, "Warning: Selected Port [{}] is not available!".format(self.ControlsWin.ui1.cBox_Port.currentText()))

        
    ###########################################################################
    # Stops the acquisition of the selected serial port
    ###########################################################################    
    def stop(self):
        
        # This function is connected to the clicked signal of the Stop button.
        self.ControlsWin.ui1.infostatus.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.ControlsWin.ui1.infostatus.setText("<font color=#000000 > Program Status Stanby</font>")
        self.ControlsWin.ui1.infobar.setText("<font color=#0000ff > Infobar </font>")
        self.InfoWin.ui3.inforef1.setText("<font color=#0000ff > Ref. Frequency </font>")               
        self.InfoWin.ui3.inforef2.setText("<font color=#0000ff > Ref. Dissipation </font>")
        print("")
        print(TAG, "Clicked STOP")
        Log.i(TAG, "Clicked STOP")
        self._timer_plot.stop()
        self._enable_ui(True)
        self.worker.stop()
    
    ###########################################################################
    # Overrides the QTCloseEvent,is connected to the close button of the window
    ###########################################################################      
    def closeEvent(self, evnt):
        
        #:param evnt: QT evnt.
        if self.worker.is_running():
          print(TAG, 'Window closed without stopping the capture, application will stop...')
          Log.i(TAG, "Window closed without stopping the capture, application will stop...")
          self.stop()
          #self.ControlsWin.close()
          #self.PlotsWin.close()
          #self.InfoWin.close()
          #evnt.accept()
    
          
    ###########################################################################
    # Enables or disables the UI elements of the window.
    ########################################################################### 
    def _enable_ui(self, enabled):
        
        #:param enabled: The value to be set for the UI elements :type enabled: bool
        self.ControlsWin.ui1.cBox_Port.setEnabled(enabled)
        self.ControlsWin.ui1.cBox_Speed.setEnabled(enabled)
        self.ControlsWin.ui1.pButton_Start.setEnabled(enabled)
        self.ControlsWin.ui1.chBox_export.setEnabled(enabled)
        self.ControlsWin.ui1.cBox_Source.setEnabled(enabled)
        self.ControlsWin.ui1.pButton_Stop.setEnabled(not enabled)
        self.ControlsWin.ui1.sBox_Samples.setEnabled(not enabled) #insert
        self.ControlsWin.ui1.pButton_Clear.setEnabled(not enabled)
        self.ControlsWin.ui1.pButton_Reference.setEnabled(not enabled)


    ###########################################################################
    # Configures specific elements of the PyQtGraph plots.
    ########################################################################### 
    def _configure_plot(self):
        
        #----------------------------------------------------------------------
        # set background color 
        self.PlotsWin.ui2.plt.setBackground(background='#0e1c20')
        self.PlotsWin.ui2.pltB.setBackground(background='#0e1c20')                          
        
        #----------------------------------------------------------------------
        # defines the graph title
        title1 = "Real-Time Plot: Amplitude / Phase"
        title2 = "Real-Time Plot: Resonance Frequency / Dissipation"
        title3 = "Real-Time Plot: Temperature"
        #--------------------------------------------------------------------------------------------------------------
        # Configures elements of the PyQtGraph plots: amplitude
        self.PlotsWin.ui2.plt.setAntialiasing(True)
        self.PlotsWin.ui2.pltB.setAntialiasing(True)
        self._plt0 = self.PlotsWin.ui2.plt.addPlot(row=0, col=1, title= title1, **{'font-size':'10pt'})
        self._plt0.showGrid(x=True, y=True)
        self._plt0.setLabel('bottom', 'Frequency', units='Hz')
        self._plt0.setLabel('left', 'Amplitude', units='dB', color=Constants.plot_colors[0], **{'font-size':'10pt'})
        
        '''
        # Configures elements of the PyQtGraph plots: phase
        self._plt1 = self.u2.plt.addPlot(row=1, col=1, title= "Real-Time Plot: Phase", **{'font-size':'12pt'})
        self._plt1.showGrid(x=True, y=True) 
        self._plt1.setLabel('bottom', 'Samples', units='n')
        self._plt1.setLabel('left', 'Phase', units='deg')
        '''
        #--------------------------------------------------------------------------------------------------------------
        # Configures elements of the PyQtGraph plots: Multiple Plot amplitude and phase
        self._plt1 = pg.ViewBox()  
        self._plt0.showAxis('right')
        self._plt0.scene().addItem(self._plt1)
        self._plt0.getAxis('right').linkToView(self._plt1)
        self._plt1.setXLink(self._plt0)
        self._plt0.enableAutoRange(axis= 'y', enable = True)
        self._plt1.enableAutoRange(axis= 'y', enable = True)
        self._plt0.setLabel('right', 'Phase', units='deg', color=Constants.plot_colors[1], **{'font-size':'10pt'})
        
        #--------------------------------------------------------------------------------------------------------------
        # Configures elements of the PyQtGraph plots: resonance
        self._yaxis = AxisItem(orientation='left')  #NonScientificAxis(orientation='left')#
        #self._yaxis.setTickSpacing(levels=[(280, 0),(25, 0), (10, 0)]) #(20,1, None)
        self._xaxis = DateAxis(orientation='bottom')
        self._plt2 = self.PlotsWin.ui2.pltB.addPlot(row=0, col=2, title= title2, **{'font-size':'12pt'}, axisItems={"bottom":self._xaxis, "left":self._yaxis})
        self._plt2.showGrid(x=True, y=True) 
        self._plt2.setLabel('bottom', 'Time',units='s')
        self._plt2.setLabel('left', 'Resonance Frequency', units='Hz', color=Constants.plot_colors[2], **{'font-size':'10pt'})
        
        #--------------------------------------------------------------------------------------------------------------
        # Configures elements of the PyQtGraph plots: Multiple Plot resonance frequency and dissipation
        self._plt3 = pg.ViewBox()  
        self._plt2.showAxis('right')
        self._plt2.scene().addItem(self._plt3)
        self._plt2.getAxis('right').linkToView(self._plt3)
        self._plt3.setXLink(self._plt2)
        self._plt2.enableAutoRange(axis= 'y', enable = True) 
        self._plt3.enableAutoRange(axis= 'y', enable = True)
        self._plt2.setLabel('bottom', 'Time',units='s')
        self._plt2.setLabel('right', 'Dissipation', units='', color=Constants.plot_colors[3], **{'font-size':'10pt'})
        '''
        # indipentent plots
        # Configures elements of the PyQtGraph plots: Q-factor/dissipation
        self._plt3 = self.PlotsWin.ui2.plt.addPlot(row=1, col=2, title="Real-Time Plot: Dissipation", axisItems={'bottom': DateAxis(orientation='bottom')})
        self._plt3.showGrid(x=True, y=True) 
        self._plt3.setLabel('bottom', 'Time (hh:mm:ss)',units='')
        self._plt3.setLabel('left', 'Dissipation', units='')
        '''
        #---------------------------------------------------------------------------------------------------------------
        # Configures text comment and info
        self._plt2.clear()
        self._text1 = pg.TextItem('',(255, 255, 255), anchor=(0.5, 0.5))
        self._text1.setHtml("<span style='font-size: 14pt'>Welcome to the real-time openQCM Q-1 device GUI </span>")
        self._text2 = pg.TextItem('',(250, 250, 250), anchor=(0.5, 0.5))
        self._text2.setHtml("<span style='font-size: 10pt'>Please do not forget, if needed, to calibrate (in air) your device with the properly selected quartz before starting. </span>")
        self._text3 = pg.TextItem('',(250, 250, 250), anchor=(0.5, 0.5))
        self._text3.setHtml("<span style='font-size: 10pt'>Please check the command line on the side for useful information. </span>")
        self._text4 = pg.TextItem('',(250, 250, 250), anchor=(0.5, 0.5))
        self._text4.setHtml("<span style='font-size: 10pt'>(https://openqcm.com - info@openqcm.com) </span>")
        self._text1.setPos(0.5,0.6)
        self._text2.setPos(0.5,0.3)
        self._text3.setPos(0.5,0.2)
        self._text4.setPos(0.5,0.1)
        self._plt2.addItem(self._text1, ignoreBounds=True)
        self._plt2.addItem(self._text2, ignoreBounds=True)
        self._plt2.addItem(self._text3, ignoreBounds=True)
        self._plt2.addItem(self._text4, ignoreBounds=True)
        #######################
        #self._lr = pg.LinearRegionItem(orientation=pg.LinearRegionItem.Horizontal)
        #self._inf1 = pg.InfiniteLine(movable=True, angle=0, label='x={value:0.2f}', 
        #             labelOpts={'color': (100,200,100), 'fill': (200,200,200,50), 'movable': True})
        #######################
        #-----------------------------------------------------------------------------------------------------------------
        # Configures elements of the PyQtGraph plots: temperature
        self._plt4 = self.PlotsWin.ui2.plt.addPlot(row=0, col=3, title= title3, axisItems={'bottom': DateAxis(orientation='bottom')})
        self._plt4.showGrid(x=True, y=True) 
        self._plt4.setLabel('bottom', 'Time',units='s')
        self._plt4.setLabel('left', 'Temperature', units='°C', color=Constants.plot_colors[4], **{'font-size':'10pt'})
      
        
    ###########################################################################
    # Configures specific elements of the QTimers
    ########################################################################### 
    def _configure_timers(self):
        
        self._timer_plot = QtCore.QTimer(self)
        #self._timer_plot.timeout.connect(self._update_plot) #moved to start method

    
    ###########################################################################
    # Configures the connections between signals and UI elements
    ########################################################################### 
    def _configure_signals(self):
        
        self.ControlsWin.ui1.pButton_Start.clicked.connect(self.start)
        self.ControlsWin.ui1.pButton_Stop.clicked.connect(self.stop)
        self.ControlsWin.ui1.pButton_Clear.clicked.connect(self.clear)
        self.ControlsWin.ui1.pButton_Reference.clicked.connect(self.reference)
        self.ControlsWin.ui1.sBox_Samples.valueChanged.connect(self._update_sample_size)
        self.ControlsWin.ui1.cBox_Source.currentIndexChanged.connect(self._source_changed)
        #--------
        self.InfoWin.ui3.pButton_Download.clicked.connect(self.start_download)
    
    ###########################################################################
    # Updates the sample size of the plot (now not used)
    ########################################################################### 
    def _update_sample_size(self):
        
        # This function is connected to the valueChanged signal of the sample Spin Box.
        if self.worker is not None:
            #Log.i(TAG, "Changing sample size")
            self.worker.reset_buffers(self.ControlsWin.ui1.sBox_Samples.value())

    
    ###########################################################################
    # Updates and redraws the graphics in the plot.
    ########################################################################### 
    def _update_plot(self):
        
        # This function is connected to the timeout signal of a QTimer
        self.worker.consume_queue1()
        self.worker.consume_queue2()
        self.worker.consume_queue3()
        self.worker.consume_queue4()
        self.worker.consume_queue5() 
        self.worker.consume_queue6() 
        
        # MEASUREMENT: dynamic frequency and dissipation labels at run-time
        ###################################################################
        if  self._get_source() == SourceType.serial:
            vector1 = self.worker.get_d1_buffer()
            vector2 = self.worker.get_d2_buffer()
            vectortemp = self.worker.get_d3_buffer()
            self._ser_error1,self._ser_error2, self._ser_control,self._ser_err_usb = self.worker.get_ser_error()
            #print(self._ser_err_usb, end='\r')
            #if self._ser_err_usb <=1:
            if vector1.any:
               # progressbar
               if self._ser_control<=Constants.environment:
                  self._completed = self._ser_control*2

               if str(vector1[0])=='nan' and not self._ser_error1 and not self._ser_error2:
                  label1 = 'processing...'
                  label2 = 'processing...'
                  label3 = 'processing...' 
                  labelstatus = 'Processing'
                  self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ffff00; padding: 1px; border: 1px solid #cccccc') #ff8000
                  color_err = '#000000'   
                  labelbar = 'Please wait, processing early data...'

               elif (str(vector1[0])=='nan' and (self._ser_error1 or self._ser_error2)):
                      if self._ser_error1 and self._ser_error2:
                        label1= ""
                        label2= ""
                        label3= ""
                        labelstatus = 'Warning'
                        color_err = '#ff0000'
                        labelbar = 'Warning: unable to apply half-power bandwidth method, lower and upper cut-off frequency not found'
                        self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
                      elif self._ser_error1:
                        label1= ""
                        label2= ""
                        label3= ""
                        labelstatus = 'Warning'
                        color_err = '#ff0000'
                        labelbar = 'Warning: unable to apply half-power bandwidth method, lower cut-off frequency (left side) not found'
                        self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
                      elif self._ser_error2:
                        label1= ""
                        label2= ""
                        label3= ""
                        labelstatus = 'Warning'
                        color_err = '#ff0000'
                        labelbar = 'Warning: unable to apply half-power bandwidth method, upper cut-off frequency (right side) not found'
                        self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
               else:
                  if not self._ser_error1 and not self._ser_error2:
                      if not self._reference_flag:
                          d1=float("{0:.2f}".format(vector1[0]))
                          d2=float("{0:.4f}".format(vector2[0]*1e6))
                          d3=float("{0:.2f}".format(vectortemp[0]))
                      else:
                          a1= vector1[0]-self._reference_value_frequency
                          a2= vector2[0]-self._reference_value_dissipation
                          d1=float("{0:.2f}".format(a1))
                          d2=float("{0:.4f}".format(a2*1e6))
                          d3=float("{0:.2f}".format(vectortemp[0])) 
                      label1= str(d1)+ " Hz"
                      label2= str(d2)+ "e-06"
                      label3= str(d3)+ " °C" 
                      labelstatus = 'Monitoring'
                      color_err = '#000000'
                      labelbar = 'Monitoring!'
                      self.ControlsWin.ui1.infostatus.setStyleSheet('background: #00ff72; padding: 1px; border: 1px solid #cccccc')
                  else:
                      if self._ser_error1 and self._ser_error2:
                        label1= "-"
                        label2= "-"
                        label3= "-"
                        labelstatus = 'Warning'
                        color_err = '#ff0000'
                        labelbar = 'Warning: unable to apply half-power bandwidth method, lower and upper cut-off frequency not found'
                        self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
                      elif self._ser_error1:
                        label1= "-"
                        label2= "-"
                        label3= "-"
                        labelstatus = 'Warning'
                        color_err = '#ff0000'
                        labelbar = 'Warning: unable to apply half-power bandwidth method, lower cut-off frequency (left side) not found'
                        self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
                      elif self._ser_error2:
                        label1= "-"
                        label2= "-"
                        label3= "-"
                        labelstatus = 'Warning'
                        color_err = '#ff0000'
                        labelbar = 'Warning: unable to apply half-power bandwidth method, upper cut-off frequency (right side) not found'
                        self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
                         
               self.InfoWin.ui3.l6a.setText("<font color=#0000ff > Temperature </font>" + label3) 
               self.InfoWin.ui3.l6.setText("<font color=#0000ff > Dissipation </font>" + label2)
               self.InfoWin.ui3.l7.setText("<font color=#0000ff > Resonance Frequency </font>" + label1)
               self.ControlsWin.ui1.infostatus.setText("<font color=#000000 > Program Status </font>" + labelstatus)
               self.ControlsWin.ui1.infobar.setText("<font color=#0000ff> Infobar </font><font color={}>{}</font>".format(color_err,labelbar))
               # progressbar 
               self.ControlsWin.ui1.progressBar.setValue(self._completed+2)
            
            #elif self._ser_err_usb >1:
                # PopUp.warning(self, Constants.app_title, "Warning: USB cable device disconnected!")  
                # self.stop() 
        
        # CALIBRATION: dynamic info in infobar at run-time
        ##################################################
        elif self._get_source() == SourceType.calibration:
            # flag for terminating calibration
            stop_flag=0
            self.ControlsWin.ui1.pButton_Stop.setEnabled(False)
            vector1 = self.worker.get_value1_buffer()
            # vector2[0] and vector3[0] flag error
            vector2 = self.worker.get_t3_buffer()
            vector3 = self.worker.get_d3_buffer()
            #print(vector1[0],vector2[0],vector3[0])
            label1 = 'not available'
            label2 = 'not available'
            label3 = 'not available' 
            labelstatus = 'Calibration Processing'
            color_err = '#000000'
            labelbar = 'The operation might take just over a minute to complete... please wait...'
            self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ffff00; padding: 1px; border: 1px solid #cccccc')
            
            # progressbar
            error1,error2,error3,self._ser_control = self.worker.get_ser_error()
            if self._ser_control< (Constants.calib_sections):
                      self._completed = (self._ser_control/(Constants.calib_sections))*100
            # calibration buffer empty
            #if vector1[0]== 0 and vector3[0]==1:
            if error1== 1 and vector3[0]==1:
              label1 = 'not available'
              label2 = 'not available'
              label3 = 'not available' 
              color_err = '#ff0000'
              labelstatus = 'Calibration Warning'
              self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
              labelbar = 'Calibration Warning: empty buffer! Please, repeat the Calibration after disconnecting/reconnecting Device!'
              stop_flag=1
            # calibration buffer empty and ValueError from the serial port 
            #elif vector1[0]== 0 and vector2[0]==1:
            elif error1== 1 and vector2[0]==1:   
              label1 = 'not available'
              label2 = 'not available'
              label3 = 'not available' 
              color_err = '#ff0000'
              labelstatus = 'Calibration Warning'
              self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
              labelbar = 'Calibration Warning: empty buffer/ValueError! Please, repeat the Calibration after disconnecting/reconnecting Device!'
              stop_flag=1 
            # calibration buffer not empty
            #elif vector1[0]!= 0:
            elif error1==0:
              label1 = 'not available'
              label2 = 'not available'
              label3 = 'not available' 
              labelstatus = 'Calibration Processing'
              color_err = '#000000'
              labelbar = 'The operation might take just over a minute to complete... please wait...'
              if vector2[0]== 0 and vector3[0]== 0:
                 labelstatus = 'Calibration Success'
                 self.ControlsWin.ui1.infostatus.setStyleSheet('background: #00ff72; padding: 1px; border: 1px solid #cccccc')
                 color_err = '#000000' 
                 labelbar = 'Calibration Success for baseline correction!' 
                 stop_flag=1
                 #print(self._k) #progressbar value 143
              elif vector2[0]== 1 or vector3[0]== 1:
                 color_err = '#ff0000'
                 labelstatus = 'Calibration Warning'
                 self.ControlsWin.ui1.infostatus.setStyleSheet('background: #ff0000; padding: 1px; border: 1px solid #cccccc')
                 if vector2[0]== 1:
                   labelbar = 'Calibration Warning: ValueError! Please, repeat the Calibration after disconnecting/reconnecting Device!'
                   stop_flag=1 ##
                 elif vector3[0]== 1: 
                   labelbar = 'Calibration Warning: Error during Peak Detection, incompatible peaks number or frequencies! Please, repeat the Calibration!'
                   stop_flag=1 ##

            self.InfoWin.ui3.l6a.setText("<font color=#0000ff>  Dissipation </font>" + label3) 
            self.InfoWin.ui3.l6.setText("<font color=#0000ff>  Dissipation </font>" + label2)
            self.InfoWin.ui3.l7.setText("<font color=#0000ff>  Resonance Frequency </font>" + label1)
            self.ControlsWin.ui1.infostatus.setText("<font color=#000000> Program Status </font>" + labelstatus)  
            self.ControlsWin.ui1.infobar.setText("<font color=#0000ff> Infobar </font><font color={}>{}</font>".format(color_err,labelbar)) 
            # progressbar -------------
            self.ControlsWin.ui1.progressBar.setValue(self._completed+10)
            
            # terminate the  calibration (simulate clicked stop)
            if stop_flag == 1:
               self._timer_plot.stop()
               self._enable_ui(True)
               self.worker.stop()                    
        '''
        # Amplitude plot
        self._plt0.clear()
        #self._plt0.plot(list(self._xdict.keys()),self.worker.get_value1_buffer(),pen=Constants.plot_colors[0])
        self._plt0.plot(self.worker.get_value1_buffer(),pen=Constants.plot_colors[0])
        
        # Phase plot
        self._plt1.clear()
        self._plt1.plot(self.worker.get_value2_buffer(),pen=Constants.plot_colors[1])
        '''
        ############################################################################################################################
        # REFERENCE SET 
        ############################################################################################################################
        if self._reference_flag:
            self._plt2.setLabel('left', 'Resonance Frequency', units='Hz', color=Constants.plot_colors[6], **{'font-size':'10pt'})
            self._plt2.setLabel('right', 'Dissipation', units='', color=Constants.plot_colors[7], **{'font-size':'10pt'})
            self.InfoWin.ui3.inforef1.setText("<font color=#0000ff > Ref. Frequency </font>" + self._labelref1)               
            self.InfoWin.ui3.inforef2.setText("<font color=#0000ff > Ref. Dissipation </font>" + self._labelref2)
            
            ###################################################################
            # Amplitude and phase multiple Plot
            def updateViews1():
                self._plt0.clear()
                self._plt1.clear()
                self._plt1.setGeometry(self._plt0.vb.sceneBoundingRect())
                self._plt1.linkedViewChanged(self._plt0.vb, self._plt1.XAxis)
            
            # updates for multiple plot y-axes
            updateViews1()
            self._plt0.vb.sigResized.connect(updateViews1) 
            self._plt0.plot(x=self._readFREQ,y=self.worker.get_value1_buffer(),pen=Constants.plot_colors[0])
            self._plt1.addItem(pg.PlotCurveItem(x=self._readFREQ,y=self.worker.get_value2_buffer(),pen=Constants.plot_colors[1]))
            
            ###################################################################
            # Resonance frequency and dissipation multiple Plot
            def updateViews2():
                self._plt2.clear()
                self._plt3.clear()
                self._plt3.setGeometry(self._plt2.vb.sceneBoundingRect())
                self._plt3.linkedViewChanged(self._plt2.vb, self._plt3.XAxis)
            
            # updates for multiple plot y-axes
            updateViews2()
            self._plt2.vb.sigResized.connect(updateViews2) 
            self._vector_1 = np.array(self.worker.get_d1_buffer())-self._reference_value_frequency          
            self._plt2.plot(x=self.worker.get_t1_buffer(),y=self._vector_1,pen=Constants.plot_colors[6]) #2
            self._vector_2 = np.array(self.worker.get_d2_buffer())-self._reference_value_dissipation 
            #Prevent the user from zooming/panning out of this specified region
            if self._get_source() == SourceType.serial:
               #dy = [(value, str(value)) for value in (range(int(min(self._readFREQ)), int(max(self._readFREQ)+1)))]
               #self._yaxis.setTicks([dy])
               #tickBottom = {self._readFREQ[250]:self._readFREQ[0],self._readFREQ[-1]:self._readFREQ[250]}
               #self._yaxis.setTicks([tickBottom.items()])
               self._plt2.setLimits(yMax=self._vector_reference_frequency[-1],yMin=self._vector_reference_frequency[0], minYRange=5)
               self._plt3.setLimits(yMax=self._vector_reference_dissipation[-1],yMin=self._vector_reference_dissipation[0], minYRange=1e-7) 
               self._plt4.setLimits(yMax=50,yMin=-10)
            self._plt3.addItem(pg.PlotCurveItem(self.worker.get_t2_buffer(),self._vector_2,pen=Constants.plot_colors[7]))
            
            ###################################################################
            # Temperature plot
            self._plt4.clear() 
            self._plt4.plot(x= self.worker.get_t3_buffer(),y=self.worker.get_d3_buffer(),pen=Constants.plot_colors[4])
            
        ###########################################################################################################################
        # REFERENCE NOT SET 
        ###########################################################################################################################  
        else:   
            self._plt2.setLabel('left', 'Resonance Frequency', units='Hz', color=Constants.plot_colors[2], **{'font-size':'10pt'})
            self._plt2.setLabel('right', 'Dissipation', units='', color=Constants.plot_colors[3], **{'font-size':'10pt'})
            self.InfoWin.ui3.inforef1.setText("<font color=#0000ff > Ref. Frequency </font>" + self._labelref1)               
            self.InfoWin.ui3.inforef2.setText("<font color=#0000ff > Ref. Dissipation </font>" + self._labelref2)
            
            ###################################################################
            # Amplitude and phase multiple Plot
            def updateViews1():
                self._plt0.clear()
                self._plt1.clear()
                self._plt1.setGeometry(self._plt0.vb.sceneBoundingRect())
                self._plt1.linkedViewChanged(self._plt0.vb, self._plt1.XAxis)
            # updates for multiple plot y-axes
            updateViews1()
            self._plt0.vb.sigResized.connect(updateViews1)
            #-----------------------------------
            if self._get_source() == SourceType.calibration: 
               calibration_readFREQ  = np.arange(len(self.worker.get_value1_buffer())) * (Constants.calib_fStep) + Constants.calibration_frequency_start   
               self._plt0.plot(x=calibration_readFREQ,y=self.worker.get_value1_buffer(),pen=Constants.plot_colors[0])
               self._plt1.addItem(pg.PlotCurveItem(x=calibration_readFREQ,y=self.worker.get_value2_buffer(),pen=Constants.plot_colors[1]))
            elif self._get_source() == SourceType.serial:  
               self._plt0.plot(x=self._readFREQ,y=self.worker.get_value1_buffer(),pen=Constants.plot_colors[0])
               self._plt1.addItem(pg.PlotCurveItem(x=self._readFREQ,y=self.worker.get_value2_buffer(),pen=Constants.plot_colors[1]))
            #--------------------------------------------------
            ###
            #img = pg.QtGui.QGraphicsPixmapItem(pg.QtGui.QPixmap('favicon.png'))
            #img.scale(1, -1)
            #self._plt1.addItem(img)
            
            ###################################################################
            # Resonance frequency and dissipation multiple Plot
            def updateViews2():
                self._plt2.clear()
                self._plt3.clear()
                self._plt3.setGeometry(self._plt2.vb.sceneBoundingRect())
                self._plt3.linkedViewChanged(self._plt2.vb, self._plt3.XAxis)
            # updates for multiple plot y-axes
            updateViews2()
            #text = pg.TextItem(html='<div style="text-align: center"><span style="color: #ff0000;"> %s</span></div>' % "prova",anchor=(0.0, 0.0)) 
            #text.setPos(0.0, 0.0) #text########
            #self._plt3.addItem(text)
            self._plt2.vb.sigResized.connect(updateViews2) 
            self._plt2.plot(x= self.worker.get_t1_buffer(),y=self.worker.get_d1_buffer(),pen=Constants.plot_colors[2])
            #Prevent the user from zooming/panning out of this specified region
            if self._get_source() == SourceType.serial:
               #dy = [(value, str(value)) for value in (range(int(min(self._readFREQ)), int(max(self._readFREQ)+1)))]
               #self._yaxis.setTicks([dy])
               #tickBottom = {self._readFREQ[250]:self._readFREQ[0],self._readFREQ[-1]:self._readFREQ[250]}
               #self._yaxis.setTicks([tickBottom.items()])
               self._plt2.setLimits(yMax=self._readFREQ[-1],yMin=self._readFREQ[0], minYRange=10, maxYRange=10000) 
               self._plt3.setLimits(yMax=(self._readFREQ[-1]-self._readFREQ[0])/self._readFREQ[0],yMin=0, minYRange=1e-6)
               self._plt4.setLimits(yMax=50,yMin=-10, minYRange=0.5)
            self._plt3.addItem(pg.PlotCurveItem(self.worker.get_t2_buffer(),self.worker.get_d2_buffer(),pen=Constants.plot_colors[3]))
            
            ##############################
            # Add  lines with labels
            #inf1 = pg.InfiniteLine(movable=True, angle=90, label='x={value:0.2f}', 
            #self._plt2.addItem(self._inf1)
            #self._plt2.addItem(self._lr)
            ##############################
            '''
            # Resonance frequency plot
            self._plt2.clear()
            self._plt2.plot(x= self.worker.get_t1_buffer(),y=self.worker.get_d1_buffer(),pen=Constants.plot_colors[2])
            # dissipation plot
            self._plt3.clear()
            self._plt3.plot(x= self.worker.get_t2_buffer(),y=self.worker.get_d2_buffer(),pen=Constants.plot_colors[3])
            '''
            ###################################################################
            # Temperature plot
            self._plt4.clear() 
            self._plt4.plot(x= self.worker.get_t3_buffer(),y=self.worker.get_d3_buffer(),pen=Constants.plot_colors[4])     
          
    ###########################################################################################################################################    
        
        
    ###########################################################################
    # Updates the source and depending boxes on change
    ###########################################################################  
    def _source_changed(self):
        
        # It is connected to the indexValueChanged signal of the Source ComboBox.
        if self._get_source() == SourceType.serial:
           print(TAG, "Scanning the source: {}".format(Constants.app_sources[0]))  # self._get_source().name
           Log.i(TAG, "Scanning the source: {}".format(Constants.app_sources[0]))
        elif self._get_source() == SourceType.calibration:
           print(TAG, "Scanning the source: {}".format(Constants.app_sources[1]))  # self._get_source().name
           Log.i(TAG, "Scanning the source: {}".format(Constants.app_sources[1]))
        
        # Clears boxes before adding new
        self.ControlsWin.ui1.cBox_Port.clear()
        self.ControlsWin.ui1.cBox_Speed.clear()
        
        # Gets the current source type
        source = self._get_source()
        ports = self.worker.get_source_ports(source)
        speeds = self.worker.get_source_speeds(source)

        if ports is not None:
            self.ControlsWin.ui1.cBox_Port.addItems(ports)
        if speeds is not None:
            self.ControlsWin.ui1.cBox_Speed.addItems(speeds)
        if self._get_source() == SourceType.serial:
            self.ControlsWin.ui1.cBox_Speed.setCurrentIndex(len(speeds) - 1)

    
    ###########################################################################
    # Gets the current source type
    ########################################################################### 
    def _get_source(self):
        
        #:rtype: SourceType.
        return SourceType(self.ControlsWin.ui1.cBox_Source.currentIndex())
    
    
    ###########################################################################
    # Cleans history plot 
    ###########################################################################    
    def clear(self):
        support=self.worker.get_d1_buffer()
        if support.any:
            if str(support[0])!='nan':
                print(TAG, "All Plots Cleared!", end='\r')
                self._update_sample_size()
                self._plt2.clear()
                self._plt3.clear()
                self._plt4.clear()
        
        
    ###########################################################################
    # Reference set/reset
    ###########################################################################     
    def reference(self):
        import numpy as np
        #import sys
        support=self.worker.get_d1_buffer()
        if support.any:
            if str(support[0])!='nan':
                ref_vector1 = [c for c in self.worker.get_d1_buffer() if ~np.isnan(c)]
                ref_vector2 = [c for c in self.worker.get_d2_buffer() if ~np.isnan(c)]
                self._reference_value_frequency = ref_vector1[0]
                self._reference_value_dissipation = ref_vector2[0]
                #sys.stdout.write("\033[K") #clear line
                if self._reference_flag:
                    self._reference_flag = False
                    print(TAG, "Reference reset!   ", end='\r')
                    self._labelref1 = "not set"
                    self._labelref2 = "not set"
                else:
                    self._reference_flag = True
                    d1=float("{0:.2f}".format(self._reference_value_frequency))
                    d2=float("{0:.4f}".format(self._reference_value_dissipation*1e6))
                    self._labelref1 = str(d1)+ "Hz"
                    self._labelref2 = str(d2)+ "e-06"
                    print(TAG, "Reference set!     ", end='\r')
                    self._vector_reference_frequency[:] = [s - self._reference_value_frequency for s in self._readFREQ]
                    xs = np.array(np.linspace(0, ((self._readFREQ[-1]-self._readFREQ[0])/self._readFREQ[0]), len(self._readFREQ)))
                    self._vector_reference_dissipation = xs-self._reference_value_dissipation
                    
    ###########################################################################
    # Checking internet connection               
    ###########################################################################   
    def internet_on(self):
       from urllib.request import urlopen
       try:
           url = "https://openqcm.com/shared/news.html"
           urlopen(url, timeout=10)
           return True
       except: 
           return False
       
    ########################################################################################################
    # Gets information from openQCM webpage and enables download button if new version software is available
    ########################################################################################################
    def get_web_info(self):    
        import pandas as pd
        # check if an Internet connection is active
        self._internet_connected = self.internet_on()
        # Get latest info from openQCM webpage
        c_types = {
                   '1': '1',
                   '2': '2',
                   '3': '3',}
        r_types = {
                   '1': 'A',
                   '2': 'B',
                   '3': 'C',}
        if self._internet_connected:
           color = '#00c600'
           labelweb2 = 'ONLINE'
           print (TAG,'Checking your internet connection {} '.format(labelweb2))
           tables = pd.read_html('https://openqcm.com/shared/news.html', index_col=0, header=0, match='1')
           df = tables[0]
           # create empty list of string  
           self._webinfo = ["" for x in range(len(df.columns)*len(df.index))] #len(df.columns)*len(df.index)=9
           # row acess mode to Pandas dataframe
           k=0
           for j in [1,2,3]:
              for i in [1,2,3]:
                  self._webinfo[k]= str(df.loc[r_types[str(j)], c_types[str(i)]])
                  k+=1
            # check for update
           if self._webinfo[0] == Constants.app_version:
              labelweb3 = 'last version installed!' 
           else:
              labelweb3 = 'version {} available!'.format(self._webinfo[0]) 
              self.InfoWin.ui3.pButton_Download.setEnabled(True)
        else:
           color = '#ff0000'
           labelweb2 = 'OFFLINE'
           labelweb3 = 'Offline, unable to check'
           print (TAG,'Checking your internet connection {} '.format(labelweb2)) 
           
        self.InfoWin.ui3.lweb2.setText("<font color=#0000ff > Checking your internet connection &nbsp;&nbsp;&nbsp;&nbsp;</font><font size=4 color={}>{}</font>".format(color,labelweb2)) 
        self.InfoWin.ui3.lweb3.setText("<font color=#0000ff > Software update status </font>" + labelweb3)  

    ########################################################################### 
    # Opens webpage for download
    ###########################################################################
    def start_download(self):
        import webbrowser
        url_download = 'https://openqcm.com/shared/q-1/openQCM_Q-1_py_v{}.zip '.format(self._webinfo[0]) 
        webbrowser.open(url_download)
        
       



from PyQt5 import QtCore, QtGui, QtWidgets
from pyqtgraph import GraphicsLayoutWidget

class Ui_Controls(object): #QtWidgets.QMainWindow
    
    def setupUi(self, MainWindow1):
        MainWindow1.setObjectName("MainWindow1")
        #MainWindow1.setGeometry(50, 50, 975, 70)
        # MainWindow1.setFixedSize(980, 150)
        #MainWindow1.resize(550, 50)
        MainWindow1.setMinimumSize(QtCore.QSize(980, 150))
        MainWindow1.resize(980, 150)
        MainWindow1.setStyleSheet("")
        MainWindow1.setTabShape(QtWidgets.QTabWidget.Rounded) 
        self.centralwidget = QtWidgets.QWidget(MainWindow1)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.Layout_controls = QtWidgets.QGridLayout()
        self.Layout_controls.setObjectName("Layout_controls")
               
        # frequency/quartz combobox -------------------------------------------
        self.cBox_Speed = QtWidgets.QComboBox(self.centralwidget)
        self.cBox_Speed.setEditable(False)
        self.cBox_Speed.setObjectName("cBox_Speed")
        self.Layout_controls.addWidget(self.cBox_Speed, 4, 1, 1, 1)
        
        # stop button ---------------------------------------------------------
        self.pButton_Stop = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Stop.setIcon(QtGui.QIcon(QtGui.QPixmap("stop_icon.ico"))) #.png
        self.pButton_Stop.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Stop.setObjectName("pButton_Stop")
        self.Layout_controls.addWidget(self.pButton_Stop, 3, 4, 1, 1)
        
        # COM port combobox ---------------------------------------------------
        self.cBox_Port = QtWidgets.QComboBox(self.centralwidget)
        self.cBox_Port.setEditable(False)
        self.cBox_Port.setObjectName("cBox_Port")
        self.Layout_controls.addWidget(self.cBox_Port, 2, 1, 1, 1)
        
        # Operation mode - source ---------------------------------------------
        self.cBox_Source = QtWidgets.QComboBox(self.centralwidget)
        self.cBox_Source.setObjectName("cBox_Source")   
        self.Layout_controls.addWidget(self.cBox_Source, 2, 0, 1, 1)
        
        # start button --------------------------------------------------------
        self.pButton_Start = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Start.setIcon(QtGui.QIcon(QtGui.QPixmap("start_icon.ico")))
        self.pButton_Start.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Start.setObjectName("pButton_Start")
        self.Layout_controls.addWidget(self.pButton_Start, 2, 4, 1, 1)
        
        # clear plots button --------------------------------------------------
        self.pButton_Clear = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Clear.setIcon(QtGui.QIcon(QtGui.QPixmap("clear_icon.ico")))
        self.pButton_Clear.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Clear.setObjectName("pButton_Clear")
        self.Layout_controls.addWidget(self.pButton_Clear, 2, 3, 1, 1)
        
        # reference button ----------------------------------------------------
        self.pButton_Reference = QtWidgets.QPushButton(self.centralwidget)
        #self.pButton_Reference.setIcon(QtGui.QIcon(QtGui.QPixmap("ref_icon.ico")))
        self.pButton_Reference.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Reference.setObjectName("pButton_Reference")
        self.Layout_controls.addWidget(self.pButton_Reference, 3, 3, 1, 1)
        
        # samples SpinBox -----------------------------------------------------
        self.sBox_Samples = QtWidgets.QSpinBox(self.centralwidget)
        self.sBox_Samples.setMinimum(1)
        self.sBox_Samples.setMaximum(100000)
        self.sBox_Samples.setProperty("value", 500)
        self.sBox_Samples.setObjectName("sBox_Samples")
        #self.sBox_Samples.setEnabled(False)
        self.Layout_controls.addWidget(self.sBox_Samples, 2, 2, 1, 1)
        
        # export file CheckBox ------------------------------------------------
        self.chBox_export = QtWidgets.QCheckBox(self.centralwidget)
        self.chBox_export.setEnabled(True)
        self.chBox_export.setObjectName("chBox_export")
        self.Layout_controls.addWidget(self.chBox_export, 4, 2, 1, 1)
        
        # Samples Number / History Buffer Size---------------------------------
        self.l5 = QtWidgets.QLabel()
        self.l5.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l5.setText("<font color=#ffffff > Samples Number / History Buffer Size </font>")
        self.l5.setFixedHeight(15)
        self.Layout_controls.addWidget(self.l5, 1, 2, 1, 1) 
        
        # Control Buttons------------------------------------------------------
        self.l = QtWidgets.QLabel()
        self.l.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l.setText("<font color=#ffffff > Control Buttons </font>")
        self.l.setFixedHeight(15)
        self.Layout_controls.addWidget(self.l, 1, 3, 1, 2)     
        
        # Operation Mode ------------------------------------------------------
        self.l0 = QtWidgets.QLabel()
        self.l0.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l0.setText("<font color=#ffffff >Operation Mode</font> </a>")
        self.l0.setFixedHeight(15)
        self.Layout_controls.addWidget(self.l0, 1, 0, 1, 1)
        
        # Resonance Frequency / Quartz Sensor ---------------------------------
        self.l2 = QtWidgets.QLabel()
        self.l2.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l2.setText("<font color=#ffffff > Resonance Frequency / Quartz Sensor </font>")
        self.l2.setFixedHeight(15)
        self.Layout_controls.addWidget(self.l2, 3, 1, 1, 1)
        
        # Serial COM Port -----------------------------------------------------
        self.l1 = QtWidgets.QLabel()
        self.l1.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l1.setText("<font color=#ffffff > Serial COM Port </font>")
        self.l1.setFixedHeight(15)
        self.Layout_controls.addWidget(self.l1, 1, 1, 1, 1)
        
        # openQCM logo---------------------------------------------------------
        self.l3 = QtWidgets.QLabel()
        self.l3.setAlignment(QtCore.Qt.AlignRight)
        self.Layout_controls.addWidget(self.l3, 4, 5, 1, 1)
        self.l3.setPixmap(QtGui.QPixmap("openqcm-logo.png"))
        
        # openQCM link --------------------------------------------------------
        self.l4 = QtWidgets.QLabel()
        self.Layout_controls.addWidget(self.l4, 3, 5, 1, 1) 
        def link(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.l4.linkActivated.connect(link)
        self.l4.setAlignment(QtCore.Qt.AlignRight)
        self.l4.setText('<a href="https://openqcm.com/openqcm-q-1-software"> <font size=4 color=#008EC0 >openqcm.com</font>') #&nbsp;
                        
        # info@openqcm.com Mail -----------------------------------------------
        self.lmail = QtWidgets.QLabel()
        self.Layout_controls.addWidget(self.lmail, 2, 5, 1, 1) #25 40
        def linkmail(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.lmail.linkActivated.connect(linkmail)
        self.lmail.setAlignment(QtCore.Qt.AlignRight)
        #self.lmail.setAlignment(QtCore.Qt.AlignLeft)
        self.lmail.setText('<a href="mailto:info@openqcm.com"> <font color=#008EC0 >info@openqcm.com</font>')
        
        # software user guide --------------------------------------------------------
        
        self.lg = QtWidgets.QLabel()
        self.Layout_controls.addWidget(self.lg, 1, 5, 1, 1) #30
        def link(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.lg.linkActivated.connect(link)
        self.lg.setAlignment(QtCore.Qt.AlignRight)
        self.lg.setText('<a href="https://openqcm.com/shared/q-1/openQCM_Q-1-userguide-v2.1.pdf"> <font color=#008EC0 >User Guide</font>') #&nbsp;
        #####################################
        '''
        self.ico = QtWidgets.QLabel()
        self.Layout_controls.addWidget(self.ico, 2, 5, 1, 1)
        self.title = QtWidgets.QLabel()
        def link(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.title.linkActivated.connect(link)
        self.title.setText('<a href="https://openqcm.com/openqcm-q-1-software"> <font color=#008EC0 >user guide</font>')
        self.Layout_controls.addWidget(self.title, 2, 5, 1, 1)
        self.pixmap = QtGui.QPixmap("guide.ico")
        self.ico.setPixmap(self.pixmap)
        self.ico.setAlignment(QtCore.Qt.AlignRight)
        self.title.setMinimumHeight(self.pixmap.height())
        self.title.setAlignment(QtCore.Qt.AlignRight)
        '''
        #####################################
        # Save file -----------------------------------------------------------
        self.infosave = QtWidgets.QLabel()
        self.infosave.setStyleSheet('background: #008EC0; padding: 1px;')
        #self.infosave.setAlignment(QtCore.Qt.AlignCenter) 
        self.infosave.setFixedHeight(15)
        self.infosave.setText("<font color=#ffffff > Save file </font>")
        self.Layout_controls.addWidget(self.infosave, 3, 2, 1, 1)
        
        # Program Status standby ----------------------------------------------  
        self.infostatus = QtWidgets.QLabel()
        self.infostatus.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.infostatus.setAlignment(QtCore.Qt.AlignCenter) 
        self.infostatus.setText("<font color=#000000 > Program Status standby </font>")
        self.Layout_controls.addWidget(self.infostatus, 4, 3, 1, 2)
        
        # Infobar -------------------------------------------------------------
        self.infobar = QtWidgets.QLabel()
        self.infobar.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        #self.infobar.setAlignment(QtCore.Qt.AlignCenter) 
        self.infobar.setText("<font color=#0000ff > Infobar </font>")
        self.Layout_controls.addWidget(self.infobar, 0, 0, 1, 5)
        
        # Progressbar -------------------------------------------------------------
        styleBar="""
                    QProgressBar
                    {
                     border: 0.5px solid grey;
                     border-radius: 1px;
                     text-align: center;
                    }
                     QProgressBar::chunk
                    {
                     background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:1, stop:0 rgba(182, 182, 182, 200), stop:1 rgba(209, 209, 209, 200));
                    }
                 """#background:url("openqcm-logo.png")
        self.progressBar = QtWidgets.QProgressBar()
        self.progressBar.setStyleSheet(styleBar) 
        self.progressBar.setProperty("value", 0)
        self.progressBar.setGeometry(QtCore.QRect(0, 0, 50, 10))
        self.progressBar.setObjectName("progressBar")
        self.Layout_controls.setColumnStretch(0,0)
        self.Layout_controls.setColumnStretch(1,2)
        self.Layout_controls.setColumnStretch(2,2)
        self.Layout_controls.setColumnStretch(3,2)
        self.Layout_controls.setColumnStretch(4,2)
        self.Layout_controls.addWidget(self.progressBar, 0, 5, 1, 1)
        # ---------------------------------------------------------------------
        
        self.gridLayout.addLayout(self.Layout_controls, 7, 1, 1, 1)
        MainWindow1.setCentralWidget(self.centralwidget) 

        self.retranslateUi(MainWindow1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow1)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", " Real-Time openQCM GUI - 2.1 - Setup/Control GUI"))
        MainWindow.setWindowIcon(QtGui.QIcon('favicon.ico')) #.png
        self.pButton_Stop.setText(_translate("MainWindow", " STOP"))
        self.pButton_Start.setText(_translate("MainWindow", "START"))
        self.pButton_Clear.setText(_translate("MainWindow", "Clear Plots"))
        self.pButton_Reference.setText(_translate("MainWindow", "Set/Reset Reference"))
        self.sBox_Samples.setSuffix(_translate("MainWindow", " / 180 min"))
        self.sBox_Samples.setPrefix(_translate("MainWindow", ""))
        self.chBox_export.setText(_translate("MainWindow", "Txt Export Sweep File"))


#######################################################################################################################

class Ui_Plots(object): 
    def setupUi(self, MainWindow2):
        MainWindow2.setObjectName("MainWindow2")
        #MainWindow2.setGeometry(100, 100, 890, 750)
        #MainWindow2.setFixedSize(1091, 770)
        #MainWindow2.resize(1091, 770)
        MainWindow2.setMinimumSize(QtCore.QSize(705, 518))
        MainWindow2.setStyleSheet("")
        MainWindow2.setTabShape(QtWidgets.QTabWidget.Rounded) 
        self.centralwidget = QtWidgets.QWidget(MainWindow2)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.Layout_graphs = QtWidgets.QGridLayout()
        self.Layout_graphs.setObjectName("Layout_graphs")
        
        self.plt = GraphicsLayoutWidget(self.centralwidget)
        self.pltB = GraphicsLayoutWidget(self.centralwidget)
        
        self.plt.setAutoFillBackground(False)
        self.plt.setStyleSheet("border: 0px;")
        self.plt.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.plt.setFrameShadow(QtWidgets.QFrame.Plain)
        self.plt.setLineWidth(0)
        self.plt.setObjectName("plt")
        
        self.pltB.setAutoFillBackground(False)
        self.pltB.setStyleSheet("border: 0px;")
        self.pltB.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.pltB.setFrameShadow(QtWidgets.QFrame.Plain)
        self.pltB.setLineWidth(0)
        self.pltB.setObjectName("pltB")
        
        self.label = QtWidgets.QLabel()
        self.Layout_graphs.addWidget(self.label, 0, 0, 1, 1)
        def link1(linkStr):
            QtGui.QDesktopServices.openUrl(QtCore.QUrl(linkStr))
        self.label.linkActivated.connect(link1)
        self.label.setText('<a href="https://openqcm.com/"> <font color=#000000 >Open-source Python application for displaying, processing and storing real-time data from openQCM Q-1 Device</font> </a>')
        
        self.Layout_graphs.addWidget(self.plt, 1, 0, 1, 1)
        self.Layout_graphs.addWidget(self.pltB, 2, 0, 1, 1)
        
        self.gridLayout.addLayout(self.Layout_graphs, 3, 1, 1, 1)
        MainWindow2.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow2)
        QtCore.QMetaObject.connectSlotsByName(MainWindow2)

    def retranslateUi(self, MainWindow2):
        _translate = QtCore.QCoreApplication.translate
        MainWindow2.setWindowIcon(QtGui.QIcon('favicon.ico')) #.png
        MainWindow2.setWindowTitle(_translate("MainWindow2", "OPENQCM Q-1 - Real-Time Plot GUI 2.1"))


############################################################################################################

class Ui_Info(object):
    def setupUi(self, MainWindow3):
        #MainWindow3.setObjectName("MainWindow3")
        #MainWindow3.setGeometry(500, 50, 100, 500)
        #MainWindow3.setFixedSize(100, 500)
        #MainWindow3.resize(100, 500)
        #MainWindow3.setMinimumSize(QtCore.QSize(0, 0))
        MainWindow3.setStyleSheet("")
        MainWindow3.setTabShape(QtWidgets.QTabWidget.Rounded) 
        MainWindow3.setMinimumSize(QtCore.QSize(268, 518))
        self.centralwidget = QtWidgets.QWidget(MainWindow3)

        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        
        # Setup Information -------------------------------------------------------------------
        self.info1 = QtWidgets.QLabel()
        self.info1.setStyleSheet('background: #008EC0; padding: 1px;')
        self.info1.setText("<font color=#ffffff > Setup Information&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</font>")
        #self.info1.setFixedWidth(250)
        self.info1.setFixedHeight(15)
        self.gridLayout_2.addWidget(self.info1, 0, 0, 1, 1)
        
        # Device Setup -------------------------------------------------------------------------
        self.info1a = QtWidgets.QLabel()
        self.info1a.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info1a.setText("<font color=#0000ff > Device Setup</font>")
        #self.info1a.setFixedWidth(250)
        #self.info1a.setFixedHeight(22)
        self.gridLayout_2.addWidget(self.info1a, 1, 0, 1, 1)
        
        # Operation Mode -----------------------------------------------------------------------
        self.info11 = QtWidgets.QLabel()
        self.info11.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info11.setText("<font color=#0000ff > Operation Mode </font>")
        #self.info11.setFixedWidth(250)
        self.gridLayout_2.addWidget(self.info11, 2, 0, 1, 1)
        
        # Data Information ---------------------------------------------------------------------
        self.info = QtWidgets.QLabel()
        self.info.setStyleSheet('background: #008EC0; padding: 1px;')
        self.info.setText("<font color=#ffffff > Data Information&nbsp;</font>")
        #self.info.setFixedWidth(250)
        self.info.setFixedHeight(15)                  
        self.gridLayout_2.addWidget(self.info, 3, 0, 1, 1)
        
        # Selected Frequency -------------------------------------------------------------------
        self.info2 = QtWidgets.QLabel()
        self.info2.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info2.setText("<font color=#0000ff > Selected Frequency </font>")
        #self.info2.setFixedWidth(250)                  
        self.gridLayout_2.addWidget(self.info2, 4, 0, 1, 1)
        
        # Frequency Value ----------------------------------------------------------------------
        self.info6 = QtWidgets.QLabel()
        self.info6.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info6.setText("<font color=#0000ff > Frequency Value </font>")
        #self.info6.setFixedWidth(250)                  
        self.gridLayout_2.addWidget(self.info6, 5, 0, 1, 1)
        
        # Start Frequency ----------------------------------------------------------------------
        self.info3 = QtWidgets.QLabel()
        self.info3.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info3.setText("<font color=#0000ff > Start Frequency </font>")
        #self.info3.setFixedWidth(250)                   
        self.gridLayout_2.addWidget(self.info3, 6, 0, 1, 1)
        
        # Stop Frequency -----------------------------------------------------------------------
        self.info4 = QtWidgets.QLabel()
        self.info4.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info4.setText("<font color=#0000ff > Stop Frequency </font>")
        #self.info4.setFixedWidth(250)                   
        self.gridLayout_2.addWidget(self.info4, 7, 0, 1, 1)
        
        # Frequency Range----------------------------------------------------------------------
        self.info4a = QtWidgets.QLabel()
        self.info4a.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info4a.setText("<font color=#0000ff > Frequency Range </font>")
        #self.info4a.setFixedWidth(250)                   
        self.gridLayout_2.addWidget(self.info4a, 8, 0, 1, 1)
        
        # Sample Rate----------------------------------------------------------------------
        self.info5 = QtWidgets.QLabel()
        self.info5.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info5.setText("<font color=#0000ff > Sample Rate </font>")
        #self.info5.setFixedWidth(250)                   
        self.gridLayout_2.addWidget(self.info5, 9, 0, 1, 1)
        
        # Sample Number----------------------------------------------------------------------
        self.info7 = QtWidgets.QLabel()
        self.info7.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.info7.setText("<font color=#0000ff > Sample Number </font>")
        #self.info7.setFixedWidth(250)                 
        self.gridLayout_2.addWidget(self.info7, 10, 0, 1, 1)
        
        # Reference Settings -------------------------------------------------------------------
        self.inforef = QtWidgets.QLabel()
        self.inforef.setStyleSheet('background: #008EC0; padding: 1px;')
        #self.inforef1.setAlignment(QtCore.Qt.AlignCenter) 
        self.inforef.setText("<font color=#ffffff > Reference Settings </font>")
        self.inforef.setFixedHeight(15)
        #self.inforef.setFixedWidth(250)                      
        self.gridLayout_2.addWidget(self.inforef, 11, 0, 1, 1)
        
        # Ref. Frequency -----------------------------------------------------------------------
        self.inforef1 = QtWidgets.QLabel()
        self.inforef1.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        #self.inforef1.setAlignment(QtCore.Qt.AlignCenter) 
        self.inforef1.setText("<font color=#0000ff > Ref. Frequency </font>")
        #self.inforef1.setFixedWidth(250)
        self.gridLayout_2.addWidget(self.inforef1, 12, 0, 1, 1)
        # Ref. Dissipation -----------------------------------------------------------------------
        
        self.inforef2 = QtWidgets.QLabel()
        self.inforef2.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        #self.inforef2.setAlignment(QtCore.Qt.AlignCenter) 
        self.inforef2.setText("<font color=#0000ff > Ref. Dissipation </font>")
        #self.inforef2.setFixedWidth(250)                     
        self.gridLayout_2.addWidget(self.inforef2, 13, 0, 1, 1)
        
        # Current Data ---------------------------------------------------------------------------
        self.l8 = QtWidgets.QLabel()
        self.l8.setStyleSheet('background: #008EC0; padding: 1px;')
        self.l8.setText("<font color=#ffffff > Current Data </font>")
        self.l8.setFixedHeight(15)                
        #self.l8.setFixedWidth(250)
        self.gridLayout_2.addWidget(self.l8, 14, 0, 1, 1) 
        
        # Resonance Frequency -------------------------------------------------------------------
        self.l7 = QtWidgets.QLabel()
        self.l7.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.l7.setText("<font color=#0000ff >  Resonance Frequency </font>")
        #self.l7.setAlignment(QtCore.Qt.AlignCenter) 
        #self.l7.setFixedWidth(250)
        self.gridLayout_2.addWidget(self.l7, 15, 0, 1, 1) 
        
        # Dissipation ---------------------------------------------------------------------------
        self.l6 = QtWidgets.QLabel()
        self.l6.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.l6.setText("<font color=#0000ff > Dissipation  </font>")
        #self.l6.setFixedWidth(250)                
        self.gridLayout_2.addWidget(self.l6, 16, 0, 1, 1) 
        
        # Temperature ---------------------------------------------------------------------------
        self.l6a = QtWidgets.QLabel()
        self.l6a.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.l6a.setText("<font color=#0000ff >  Temperature </font>")
        #self.l6a.setFixedWidth(250)                
        self.gridLayout_2.addWidget(self.l6a, 17, 0, 1, 1)
     
        # Info from openQCM website -------------------------------------------------------------
        self.lweb = QtWidgets.QLabel()
        self.lweb.setStyleSheet('background: #008EC0; padding: 1px;')
        self.lweb.setText("<font color=#ffffff > Info from openQCM Website </font>")
        self.lweb.setFixedHeight(15)                
        #self.lweb.setFixedWidth(250)
        self.gridLayout_2.addWidget(self.lweb, 18, 0, 1, 1) 
        
        # Check internet connection -------------------------------------------------------------
        self.lweb2 = QtWidgets.QLabel()
        self.lweb2.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.lweb2.setText("<font color=#0000ff > Checking your internet connection </font>")
        # self.lweb2.setFixedHeight(20)                
        #self.lweb2.setFixedWidth(250)
        self.gridLayout_2.addWidget(self.lweb2, 19, 0, 1, 1)   
        
        # Software update status ----------------------------------------------------------------
        self.lweb3 = QtWidgets.QLabel()
        self.lweb3.setStyleSheet('background: white; padding: 1px; border: 1px solid #cccccc')
        self.lweb3.setText("<font color=#0000ff > Software update status </font>")
        # self.lweb3.setFixedHeight(16)                
        #self.lweb3.setFixedWidth(300)
        self.gridLayout_2.addWidget(self.lweb3, 20, 0, 1, 1) 
        
        # dovnload button -----------------------------------------------------------------------
        self.pButton_Download = QtWidgets.QPushButton(self.centralwidget)
        self.pButton_Download.setIcon(QtGui.QIcon(QtGui.QPixmap("download_icon.ico"))) #.png
        self.pButton_Download.setMinimumSize(QtCore.QSize(0, 0))
        self.pButton_Download.setObjectName("pButton_download")
        self.pButton_Download.setFixedWidth(145)
        self.pButton_Download.setEnabled(False)
        self.gridLayout_2.addWidget(self.pButton_Download, 21, 0, 1, 1, QtCore.Qt.AlignRight)
        ##########################################################################################
        
        self.gridLayout.addLayout(self.gridLayout_2, 3, 1, 1, 1)
        MainWindow3.setCentralWidget(self.centralwidget) 

        self.retranslateUi(MainWindow3)
        QtCore.QMetaObject.connectSlotsByName(MainWindow3)

    def retranslateUi(self, MainWindow3):
        _translate = QtCore.QCoreApplication.translate
        self.pButton_Download.setText(_translate("MainWindow3", " Download zip file"))
        MainWindow3.setWindowIcon(QtGui.QIcon('favicon.ico')) #.png 
        MainWindow3.setWindowTitle(_translate("MainWindow3", "Information GUI"))





from multiprocessing import freeze_support
import sys
import os #add
from PyQt5 import QtGui
from openQCM.common.architecture import Architecture,OSType
from openQCM.common.arguments import Arguments
from openQCM.common.logger import Logger as Log
from openQCM.core.constants import MinimalPython, Constants
from openQCM.ui import mainWindow

TAG = ""#"[Application]"


###############################################################################
# Main Application
###############################################################################
class OPENQCM:
    
    ###########################################################################
    # Initializing values for application
    ###########################################################################
    def __init__(self, argv=sys.argv):
          
        freeze_support()
        self._args = self._init_logger()
        self._app = QtGui.QApplication(argv)
        ##
        if Architecture.get_os() is OSType.windows:
          '''
          # Python console position and dimensions
          import win32gui
          xpos = 10
          ypos = 10
          width = 980
          length = 510
          def enumHandler(hwnd, lParam):
             if win32gui.IsWindowVisible(hwnd):
                win32gui.MoveWindow(hwnd, xpos, ypos, width, length, True)   
          win32gui.EnumWindows(enumHandler, None)
          '''
          ## Set python console title 
          import ctypes
          ctypes.windll.kernel32.SetConsoleTitleW("Real-Time openQCM GUI - command line")
        ##
    
    ###########################################################################
    # Runs the application
    ###########################################################################
    def run(self):
        if Architecture.is_python_version(MinimalPython.major, minor=MinimalPython.minor):
            print(TAG,"Path:",os.path.dirname(__file__)) #add
            print('')
            print(TAG,"Application started")
            Log.i(TAG, "Application started")
            win = mainWindow.MainWindow(samples=self._args.get_user_samples())
            #win.setWindowTitle("{} - {}".format(Constants.app_title, Constants.app_version))
            #win.move(500, 20) #GUI position (x,y) on the screen 
            #win.show()
            self._app.exec()
            print(TAG, "Finishing Application...")
            print(TAG, "Application closed")
            Log.i(TAG, "Finishing Application...\n")
            Log.i(TAG, "Application closed\n")
            win.close()
        else:
            self._fail()
        self.close()

    ###########################################################################
    # Closes application
    ###########################################################################
    def close(self):
        self._app.exit()
        Log.close()
        sys.exit()
               
    ###########################################################################
    # Initializing logger
    ###########################################################################
    @staticmethod
    def _init_logger():
        args = Arguments()
        args.create()
        args.set_user_log_level()
        return args
    
    ###########################################################################
    # Specifies the minimal Python version required
    ###########################################################################
    @staticmethod
    def _fail():
        txt = str("Application requires Python {}.{} to run".format(MinimalPython.major, MinimalPython.minor))
        print(TAG, txt)
        Log.e(TAG, txt)


if __name__ == '__main__':
    OPENQCM().run()
